"""Utils for packages."""
import gettext
import os
import subprocess
from buzio import console
from enum import Enum

args = {
    'domain': 'messages',
    'localedir': os.path.join('..', 'locale'),
    'fallback': True
}

t = gettext.translation(**args)
locale = t.gettext
_ = locale

class AutoEnum(Enum):
    """Base class for Auto Enums."""

    def __str__(self):
        """Magic string class.

        Return:
            string: name of the selected enum
        """
        return self.name

    def _generate_next_value_(name, start, count, last_values):
        """Generate next value for auto enum.

        Return:
            string: property name
        """
        return name

    @classmethod
    def listall(cls):
        """Return all object class enums.

        Return:
            object: every class member
        """
        return [
            member for name, member in cls.__members__.items()
        ]


def run_command(
        task,
        title=None,
        get_stdout=False,
        run_stdout=False):
    if title:
        console.section(title)

    try:
        if run_stdout:
            command = subprocess.check_output(task, shell=True)

            if not command:
                print(_('An error occur. Task aborted.'))
                return False

            ret = subprocess.call(command, shell=True)

        elif get_stdout is True:
            ret = subprocess.check_output(task, shell=True)
        else:
            ret = subprocess.call(
                task,
                shell=True,
                stderr=subprocess.STDOUT
            )

        if ret != 0 and not get_stdout:
            return False
    except BaseException:
        return False

    return True if not get_stdout else ret.decode('utf-8')



"""Project module."""
import os
import re
import yaml
from buzio import console, formatStr
from enum import auto
from tabulate import tabulate
from megalus.core.utils import locale as _
from megalus.core.utils import AutoEnum


class AppType(AutoEnum):
    """Enum for Application Types."""

    Backend = auto()
    Frontend = auto()
    Database = auto()
    Cache = auto()
    Broker = auto()


class CVS(AutoEnum):
    """Enum for CVS services."""

    Github = auto()
    Bitbucket = auto()
    Other = auto()

    def get_data(self):
        """Function: get_data.

        Return API endpoints for selected CVS. If you choose
        'other' no information was given.

        Return
        ------
        dict or None
        """
        if self.name == "Github":
            data = {
                "vcs_base_url": "https://{token}@github.com/{name}",
                "vcs_api_url": "https://api.github.com",
                "vcs_repository_url": "repos/{name}/{repo}/commits/{sha}",
                "vcs_pull_request_url": "/repos/{name}/{repo}/pulls"
            }
        elif self.name == "Bitbucket":
            data = {
                "vcs_base_url": "https://{token}@bitbucket.org/{name}/",
                "vcs_api_url": "https://api.bitbucket.org/",
                "vcs_repository_url":
                "2.0/repositories/{name}/{repo}/commit/{sha}/statuses",
                "vcs_pull_request_url":
                "2.0/repositories/{name}/{repo}/pullrequests/"}
        else:
            data = None
        return data


class ProjectMixin():
    """Setting Class."""

    @property
    def has_project(self):
        """Property: has_project.

        Check if project data exists.

        Return
        ------
        Bool
        """
        return self.project is not None

    def edit_project(self):
        """Function: edit_project.

        Loads docker-compose.yml and other fields
        to populate settings.project
        """
        try:
            if not self.project:
                title = _("Create new Project")
                self.project = {
                    "dc_path": "",
                    "dc_version": "",
                    "dc_timestamp": "",
                    "name": "",
                    "env_vars": [],
                    "use_vpn": False,
                    "dc_applications": [],
                    "cvs": CVS.Github,
                    "git_token": "",
                    "use_aws": False,
                    "use_slack": False,
                    "use_ECR": False
                }
            else:
                title = _("Edit Project")
            self.helptxt = {
                "dc_path":
                    _("Full path to docker-compose.yml"),
                "name": _("Project Name"),
                "env_vars":
                    _("List all mandatory Environment "
                        "Variables (use , to separate them)"),
                "use_vpn": _("Use VPN for deploy/build?"),
                "cvs": _("CVS used"),
                "git_token": _("Your Github Personal Token or Bitbucket App Password"),
                "use_aws": _("Use AWS"),
                "use_slack": _("Use Slack"),
                "use_ECR": _("Use Amazon ECR for docker images")
            }

            console.clear()
            console.section(title, full_width=True, transform="center")

            console.section(_("1. Answer the following questions"
                              " for configure project."))

            for config_key in sorted([k for k in self.project]):
                # Get CVS
                if config_key == "cvs":
                    print(_("Control Version System utilized?") + "\n")
                    self.project[config_key] = console.choose(
                        CVS.listall(),
                        question=self.helptxt.get(
                            config_key,
                            config_key),
                        default=self.project['cvs']
                        if self.project['cvs'] else None)
                    print("\n")

                # Ask after reading docker-compose
                elif config_key == "env_vars":
                    continue

                # Ignore docker-compose fields
                elif "dc_" in config_key:
                    continue
                else:
                    # Get boolean fields
                    if isinstance(self.project[config_key], bool):
                        self.project[config_key] = console.confirm(
                            self.helptxt.get(config_key, config_key),
                            default=self.project[config_key]
                            if self.project[config_key] is not None else None
                        )

                    # Get list fields
                    elif isinstance(self.project[config_key], list):
                        ret = console.ask(
                            self.helptxt.get(config_key, config_key),
                            required=True,
                            default=self.project[config_key]
                            if self.project[config_key] else None
                        )
                        if isinstance(ret, list):
                            self.project[config_key] = ret
                        else:
                            self.project[config_key] = ret.split(",")

                    # Get and validate name field
                    elif config_key == "name":
                        self.project[config_key] = console.ask(
                            self.helptxt.get(config_key, config_key),
                            required=True, validator=self._check_name,
                            default=self.project['name']
                            if self.project['name'] else None)

                    # Other fields
                    else:
                        self.project[config_key] = console.ask(
                            self.helptxt.get(config_key, config_key),
                            required=True,
                            default=self.project[config_key]
                            if self.project[config_key] else None)

            # Parse data from compose
            self.get_data_from_compose()

            # Check for environment variables
            console.section(_("3. Define environment variables"))
            default_list = self.project["env_vars"] \
                if self.project["env_vars"] else self._get_env_vars()
            ret = console.ask(
                self.helptxt.get("env_vars", "env_vars"),
                required=True,
                default=default_list
            )
            if isinstance(ret, list):
                self.project["env_vars"] = ret
            else:
                self.project["env_vars"] = ret.split(",")
        except KeyboardInterrupt:
            return False

        # Show config and options
        self.show_project_config()

    def get_data_from_compose(self):
        """function: get_data_from_compose.

        Parse data from Docker-Compose file
        iterating each service for asking specific
        fields for them.
        """
        # Find Docker-compose.yml
        console.section(_("2. Answer the following questions"
                          " for configure containers."))
        self.project['dc_path'] = console.ask(
            self.helptxt['dc_path'],
            validator=self._validate_yaml,
            default=self.project['dc_path']
            if self.project['dc_path'] else None
        )

        # Load data from yaml
        if "docker-compose" not in self.project['dc_path']:
            dcpath = os.path.join(
                os.path.expanduser("~")
                if "~" in self.project['dc_path'] else "",
                self.project['dc_path'].replace("~", ""),
                "docker-compose.yml")
        with open(dcpath, 'r') as file:
            self.raw_data = file.read()
            self.compose = yaml.load(self.raw_data)

        # Parse data from Docker-Compose.yml
        self.project['dc_version'] = self.compose['version']
        self.project['dc_timestamp'] = os.path.getmtime(
            self.project['dc_path'])
        old_list = self.project['dc_applications']
        self.project['dc_applications'] = []

        for key in self.compose['services']:
            console.section(key, theme="info")

            # Container Name and Base
            data = self.compose['services'][key]
            name = data['container_name'] \
                if 'container_name' in data else key
            container_base = 'build' if 'build' in data else 'image'

            # Container Type
            container_type = console.choose(
                choices=AppType.listall(),
                question=_("Select Type for: '{}'").format(key),
                default=self._get_application_default(key)
            )

            # Container Links
            links = data.get('links', [])

            # Options for 'build' base containers
            if container_base == "build":
                old_data = self.get_application_data(name, old_list)
                base_first = console.confirm(
                    _("This service needs to build base image first?"),
                    default=old_data['build_base_first']
                    if old_data else None
                )

                if base_first:
                    base_path = console.ask(
                        _("Path to base image Dockerfile"),
                        required=True,
                        validator=self._find_dockerfile,
                        default=old_data['base_path'] if old_data else None
                    )
                    base_image_tag = console.ask(
                        _("Base image tag (use format 'name:tag')"),
                        required=True,
                        default=old_data['base_image_tag']
                        if old_data else None)
                else:
                    base_path = None
                    base_image_tag = None
                if data.get('build', {}).get('dockerfile', None):
                    dockerfile_dev = data['build']['dockerfile']
                else:
                    dockerfile_dev = "dockerfile"
                dockerfile_stage = console.ask(
                    _("Dockerfile name for stage"),
                    default=dockerfile_dev)
                dockerfile_deploy = console.ask(
                    _("Dockerfile name for deploy"),
                    default=dockerfile_dev)
                if data.get('build', {}).get('context', None):
                    container_path = data['build']['context']
                else:
                    container_path = data['build']

                self.project['dc_applications'].append(
                    {
                        'name': name,
                        'base': container_base,
                        'type': container_type,
                        'links': links,
                        'path': container_path,
                        'build_base_first': base_first,
                        'base_path': base_path,
                        'base_image_tag': base_image_tag,
                        'dockerfile': {
                            'dev': dockerfile_dev,
                            'stage': dockerfile_stage,
                            'deploy': dockerfile_deploy
                        }
                    }
                )
            else:
                self.project['dc_applications'].append(
                    {
                        'name': name,
                        'base': container_base,
                        'type': container_type,
                        'links': links
                    }
                )

    def show_project_config(self):
        """Function: show_project_config.

        Show project fields using tabulate
        and ask for options:
        Add, Edit, Save, Delete Project or Exit
        """
        console.section(
            _("Project Configuration: {}").format(
                self.project['name']),
            full_width=True)
        console.info(
            _("Docker-Compose path: {}\n").format(
                self.project['dc_path']), use_prefix=False)
        table_data = []
        key_list = sorted([key for key in self.project])
        for key in key_list:
            if "dc_" in key and key != "dc_applications":
                continue
            data = self.project[key]
            if isinstance(data, list):
                if key == "dc_applications":
                    for i, app_dict in enumerate(data):
                        for x, app_config in enumerate(app_dict):
                            if app_dict[app_config] is None:
                                continue
                            if isinstance(app_dict[app_config], list):
                                for z, listn in enumerate(
                                        app_dict[app_config]):
                                    table_data.append([
                                        key if x == 0 else "",
                                        app_dict['name'] if x == 0 else "",
                                        app_config if z == 0 else "",
                                        listn
                                    ])
                            elif isinstance(app_dict[app_config], dict):
                                for z, listn in enumerate(
                                        app_dict[app_config]):
                                    table_data.append([
                                        key if x == 0 else "",
                                        app_dict['name'] if x == 0 else "",
                                        app_config if z == 0 else "",
                                        "{}: {}".format(
                                            listn, app_dict[app_config][listn])
                                    ])
                            elif app_config == "base_path" \
                                    and app_dict[app_config]:
                                last_one = os.path.split(
                                    app_dict[app_config])[-1]

                                table_data.append([
                                    key if x == 0 and i == 0 else "",
                                    app_dict['name'] if x == 0 else "",
                                    app_config,
                                    "..." + last_one
                                ])
                            else:
                                table_data.append([
                                    key if x == 0 and i == 0 else "",
                                    app_dict['name'] if x == 0 else "",
                                    app_config,
                                    app_dict[app_config]
                                ])
                elif isinstance(data, list):
                    for i, item in enumerate(data):
                        table_data.append([
                            key if i == 0 else "",
                            item
                        ])
                else:
                    for i, item in enumerate(data):
                        table_data.append([
                            key if i == 0 else "",
                            formatStr(item)
                        ])
            else:
                table_data.append(
                    [key, data]
                )
        print(tabulate(
            table_data,
            headers=[_("Key"), _("Value"), _("Key"), _("Value")],
            tablefmt="presto"
        ))

        print('\n')
        try:
            ret = console.select(
                [
                    _("Add"),
                    _("Edit"),
                    _("Delete"),
                    _("Save and Exit"),
                    _("Exit")
                ],
                default=3
            )
            if ret == 0:
                self.project = None
                self.edit_project()
            elif ret == 1:
                self.edit_project()
            elif ret == 2:
                self.delete_project()
                console.success(_("Project deleted."))
                return True
            elif ret == 3:
                self.save_data()
                console.success(_("Project saved."))
                return True
            else:
                return True
        except KeyboardInterrupt:
            return False

    def change_project(self, name=None):
        """Function: change_project.

        Change current project.
        """
        if name:
            project_list = [
                self.data['projects'][key]['name']
                for key in self.data['projects']
                if name in self.data['projects'][key]['name'].lower()
            ]
        else:
            project_list = [
                self.data['projects'][key]['name']
                for key in self.data['projects']
            ]
        if not project_list:
            console.error(_("No Project Found."))
            ret = None
        elif len(project_list) == 1:
            ret = project_list[0]
        else:
            console.section(_("Change current project"))
            try:
                ret = console.choose(
                    sorted([key for key in project_list])
                )
            except KeyboardInterrupt:
                console.error(_("Operation interrupted."))
                return False
        if not ret:
            return False
        ret = formatStr.slugify(ret)
        changed = True if ret != self.data['config']['current'] else False
        self.data['config']['current'] = ret
        self.save_data()
        if changed:
            console.success(_("Current Project changed to {}").format(ret))
        else:
            console.info(_("Current project not changed"))

    def _check_name(self, name):
        names_list = [
            key for key in self.data['projects']
        ]
        name = formatStr.slugify(name)
        return name not in names_list

    def _get_application_default(self, key):
        if re.search("cache", key):
            return AppType.Cache
        elif re.search("redis|rabbit|broker", key):
            return AppType.Broker
        elif re.search("postgres|db|sql|database", key):
            return AppType.Database
        elif re.search("api|back|django|pyramid|java|flask", key):
            return AppType.Backend
        elif re.search("front|spa|angular|react", key):
            return AppType.Frontend
        else:
            return None

    def _find_dockerfile(self, path):
        testpath = os.path.join(path, 'Dockerfile')
        ret = os.path.isfile(testpath)
        if not ret:
            console.error(
                _("Dockerfile not found on: {}").format(testpath))
        return ret

    def _get_env_vars(self):

        data = re.finditer("\${(\w+)}", self.raw_data)

        env_var_list = set([
            m.group(1)
            for m in data
        ])

        return list(env_var_list)

    def get_application_data(self, name, search_list=None):
        if not search_list:
            search_list = self.project['dc_applications']
        ret = [
            app_data
            for app_data in search_list
            if app_data['name'] == name
        ]
        return ret[0] if ret else None



"""[summary].

[description]
"""
import os
from megalus.core import utils
from megalus.projects.config import profile


def run(application, port):
    """[summary].

    [description]

    Arguments:
        application {[type]} -- [description]
        port {[type]} -- [description]

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False

    container_id, name = utils.get_app(
        application=application,
        title="Rodar Telnet",
        data=data
    )

    if not container_id:
        return False

    os.chdir(data.project_path)
    os.system(
        'docker exec -ti {} telnet 127.0.0.1 {}'.format(
            container_id, port
        )
    )
    return False



"""Megalus Tool.

For help type: 'meg help'

Usage:
    meg build [<application>] [--no-cache] [--base-only]
    meg change [<project>]
    meg envs
    meg help
    meg profile
    meg project

Options:
    --help          Show this text.

"""
import re
import sys
import os
from docopt import docopt
from buzio import console
from tabulate import tabulate
from megalus import __version__
from megalus.core.utils import locale as _
from megalus.projects.settings import settings
from megalus.version import check_version
from megalus.command.docker import Build


class Command():
    """Command class."""

    def __init__(self, settings=None):
        """Function: __init__.

        Summary: InsertHere
        Examples: InsertHere
        Attributes:
            @param (self):InsertHere
            @param (project):InsertHere
            @param (profile):InsertHere
        Returns: InsertHere
        """
        self.settings = settings

    def run(self, arguments):
        """Function: run.

        Summary: InsertHere
        Examples: InsertHere
        Attributes:
            @param (self):InsertHere
            @param (arguments):InsertHere
        Returns: InsertHere
        """
        self.arguments = arguments
        matches = re.finditer(r"(meg )(\w+)", __doc__)
        command = [
            m.group(2)
            for m in matches
            if self.arguments[m.group(2)]
        ][0]

        if command:
            function = getattr(self, "_{}".format(command))
            return function()
        else:
            return False

    def _project(self):
        console.clear()
        return settings.show_project_config()

    def _profile(self):
        return settings.show_profile_config()

    def _change(self):
        return settings.change_project(name=self.arguments["<project>"])

    def _help(self):
        print("Help Placeholder")
        return True

    def _build(self):
        return Build(
            settings=self.settings,
            arguments=self.arguments
        ).run()

    def _envs(self):
        env_list = [
            [obj, os.environ.get(obj, "--")]
            for obj in self.settings.project['env_vars']
        ]
        console.section(_("Project Enviroment Variables"))
        print(tabulate(
            env_list,
            headers=[_("Variable"), _("Value")],
            tablefmt="presto"
        )
        )


def main():
    """Function: main.

    Main routine:
    1. Make several checks:
        - outdated version
        - has project
        - has profile
        - docker-compose.yml has modified
    2. Run edit_profile and/or edit_project if not settings
    3. Run apropriate command

    Return
    ------
    System Exit Code 0 or 1
    """
    # Show introduction
    console.box("Megalus v.{}".format(__version__))
    check_version()
    arguments = docopt(__doc__, version=__version__)

    # Show Help
    if arguments['help']:
        command = Command()
        command.run(arguments)
        sys.exit(0)
    else:
        console.info(
            _("For help type: 'meg help'"),
            use_prefix=False,
            theme="dark"
        )

    # Load data from files
    settings.load()

    # Check for Project
    if not settings.has_project:
        ret = console.confirm(
            "\n" + _("Project not found. Do you want create one?"),
            default=True)
        if ret:
            ret = settings.edit_project()
            if not ret:
                sys.exit(1)
        else:
            sys.exit(1)
    else:
        console.info(
            _("Current Project: {}").format(settings.project['name']),
            use_prefix=False
        )

    # If docker-compose.yml has changed
    if settings.has_project and settings.docker_compose_modified:
        ret = console.confirm(
            _("\nDocker-Compose file has changed. Do you want to update data"),
            theme="warning")
        if ret:
            settings.edit_project()
        else:
            settings.save_data()

    # Check for environment variables
    # except for profile/project commands
    if not arguments['project'] \
            and not arguments['profile'] \
            and not arguments['change']:
        if not settings.check_envs():
            sys.exit(1)

    # Check for profile
    if not settings.has_profile:
        ret = console.confirm(
            "\n" + _("Profile not found. Do you want create one?"),
            default=True)
        if ret:
            settings.edit_profile()

    # Run command if ready
    if settings.ready:
        command = Command(settings)
        ret = command.run(arguments)
    else:
        console.error(_("Configuration not ready"))
        ret = False

    if not ret:
        print('\n')
    else:
        print(_("\nOperation completed.\n"))

    sys.exit(ret)


if __name__ == "__main__":
    main()



"""[summary].

[description]
"""
import os
from megalus.core import utils
from megalus.projects.config import profile
from megalus.projects.setup import settings


def run_django_commands(cmd_list, data, name):
    """[summary].

    [description]

    Arguments:
        cmd_list {[type]} -- [description]
        data {[type]} -- [description]
        name {[type]} -- [description]
    """
    for cmd in cmd_list:
        utils.console(
            "Rodando comando: {}".format(cmd),
            style="warning",
            use_prefix=False)
        os.system(
            "cd {folder} && docker-compose {cmd} {app} {com}".format(
                folder=data['docker_compose_path'],
                cmd="run",
                app=name,
                com=cmd)
        )


def run(application):
    """[summary].

    [description]

    Arguments:
        application {[type]} -- [description]

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False

    # Selecionar Banco
    container_id, name = utils.get_app(
        application=application,
        title="Redefinir Banco de Dados",
        data=data,
        stop=True,
        no_container=True
    )

    if not name:
        return False

    dbdata = [
        obj
        for obj in settings.LOCAL_DBS
        if obj.get('admin') == name
    ]

    if not dbdata:
        utils.print_error("Banco de Dados não encontrado para a Aplicação.")
        return False

    local_name = dbdata[0].get('local_name', None)
    db_name = dbdata[0].get('name', None)
    db_user = dbdata[0].get('user', None)

    if not local_name or not db_name or not db_user:
        utils.print_error("Configuração do Banco de Dados não encontrado.")
        return False

    # Confirmar
    warning_text = "Este comando exclui o banco de "
    "dados {} - Deseja continuar".format(
        local_name.upper())
    resp = utils.confirma(warning_text, warning=True)
    if resp:
        # Parar os containers
        utils.console(
            "\nParando containers",
            style="warning",
            use_prefix=False)
        utils.stop_all(data)

        # Iniciar o Postgres
        utils.console(
            "\nIniciando {}".format(local_name),
            style="warning",
            use_prefix=False)
        new_container_id = utils.run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': "cd {} && docker-compose "
                    "run -d {}".format(
                        data['docker_compose_path'],
                        local_name),
                    'run_stdout': False
                },
            ]
        )
        new_container_id = new_container_id.replace("\n", "")

        # Apagar o banco antigo
        cmd = "dropdb -U {user} {database}".format(
            user=db_user,
            database=db_name
        )
        utils.console(
            "\nRodando comando: {}".format(cmd),
            style="warning",
            use_prefix=False)
        os.system(
            "docker exec -ti {container} {com}".format(
                folder=data['docker_compose_path'],
                container=new_container_id,
                com=cmd)
        )
        # Criar o banco novo
        cmd = "createdb -U {user} -O {user} {database}".format(
            user=db_user,
            database=db_name
        )
        utils.console(
            "\nRodando comando: {}".format(cmd),
            style="warning",
            use_prefix=False)
        os.system(
            "docker exec -ti {container} {com}".format(
                folder=data['docker_compose_path'],
                container=new_container_id,
                com=cmd)
        )
        # Reiniciar os containers
        utils.console(
            "\nReiniciando Containers".format(cmd),
            style="warning",
            use_prefix=False)
        utils.stop_all(data)
        os.system(
            "cd {folder} && docker-compose up -d".format(
                folder=data['docker_compose_path']
            )
        )
        # Rodar Operações do Django
        cmd_list = [
            "python manage.py makemigrations",
            "python manage.py migrate",
            "python manage.py makemessages"
            "python manage.py compilemessages",
            "python manage.py createdata"
        ]
        run_django_commands(cmd_list, data, name)

        # Parar os containers
        utils.console(
            "\nParando containers",
            style="warning",
            use_prefix=False)
        utils.stop_all(data)

        # Excluir containers extra
        os.system(
            "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
        )

        return True



"""Version package.

Checks version number for upgrades in PyPI
"""
import requests
from buzio import console
from distutils.version import LooseVersion
from megalus import __version__
from megalus.core.utils import locale as _

def versions():
    """Function: versions.

    Summary: Request all versions registered in PyPI
    Returns: list
    """
    url = "https://pypi.python.org/pypi/Megalus/json"
    data = None
    versions = None
    try:
        ret = requests.get(url, timeout=1)
        data = ret.json()
    except BaseException:
        pass
    if data:
        versions = list(data["releases"].keys())
        versions.sort(key=LooseVersion)
    return versions


def check_version():
    """Function: check_version.

    Summary: Compares actual version vs last known
    version in PyPI for upgrades
    Returns: Bool = true if updated
    """
    last_version = __version__
    version_data = versions()
    if version_data:
        last_version = version_data[-1]
    if LooseVersion(last_version) > LooseVersion(__version__) and \
            ("rc" not in last_version and
                "b" not in last_version and "dev" not in last_version):
        console.warning(
            _("You're running a outdated version.") + "\n" +
            _("Last Version: {}").format(last_version) + "\n"
        )
        return False
    else:
        return True




import os
from megalus.core.messages import notify
from megalus.core.utils import (
    run_command, get_app, get_compose_data, console, stop_all
)
from megalus.projects.config import profile
from megalus.projects.setup import settings


def run(application):
    data = profile.get_data()
    if not data:
        return False

    if application == "base" or not application:
        path = os.path.join(
            data.project_path,
            settings.DOCKER_BASE_IMAGE_REPO,
            'baseimage'
        )
        ret = run_command(
            title="Gera Imagem Base do Docker",
            command_list=[
                {
                    'command': "cd {} && docker build -t {}:dev .".format(
                        path,
                        settings.DOCKER_BASE_IMAGE_REPO
                    ),
                    'run_stdout': False
                },
            ]
        )
        if not ret:
            return False

        if application == "base":
            notify(msg="A operação de Build foi concluída")
            return True

    name = ""
    container_id = None
    if application:
        container_id, name = get_app(
            application=application,
            title="Build da Aplicacao",
            data=data,
            stop=True
        )
        if not container_id:
            return False

    if settings.USE_ECR:
        run_command(
            get_stdout=False,
            command_list=[
                {
                    'command': "aws ecr get-login"
                    " --region {region}".format(
                        region=data['aws_region']),
                    'run_stdout': True
                }
            ]
        )

    stop_all(data)

    console("Gerando a build dos containers", style="section")
    os.system(
        "cd {folder} && docker-compose build {app}".format(
            folder=data['docker_compose_path'],
            app=name)
    )

    stop_all(data)

    # Exclui container extra
    # docker rm $(docker ps -a | grep host_run |  awk '{print $1}')
    retdocker = run_command(
        get_stdout=True,
        command_list=[
            {
                'command': 'docker ps -a | grep _run_',
                'run_stdout': False
            }
        ]
    )
    if retdocker:
        os.system(
            "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
        )
    notify(msg="A operação de Build foi concluída")



"""Data module.

Define Data class, which is used to manipulate
configuration files
"""
import glob
import os
import yaml
import getpass
from buzio import console, formatStr
from megalus.core.utils import locale as _


class DataMixin():
    """Manipulate datafiles.

    megconfig.yml - Main configuration and Projects data
    docker-compose.yml - Data from docker compose
    <project slug>_<$USER>.yml - Project User profiles
    """

    @property
    def docker_compose_modified(self):
        """Property: docker_compose_modified.

        Check if docker-compose.yml has changed

        Return
        ------
        Bool
        """
        timestamp = os.path.getmtime(self.project['dc_path'])
        return self.project['dc_timestamp'] != timestamp

    def load(self):
        """Function: load.

        Try read project and user data from disk,
        creating folder and file if not exists.
        """
        basepath = os.path.expanduser("~")
        self.path = os.path.join(basepath, ".megalus")
        self.datapath = os.path.join(self.path, "megconfig.yml")

        # Create path
        if not os.path.exists(self.path):
            os.makedirs(self.path)

        # Load data
        if os.path.isfile(self.datapath):
            with open(self.datapath, 'r') as file:
                self.data = yaml.load(file.read())

        # Load current project
        if self.data['projects'] and self.data['config']['current']:
            self.project = \
                self.data['projects'][self.data['config']['current']]

        # Load user
        if self.project:
            self.profilepath = self._get_profile_path()
            if os.path.isfile(self.profilepath):
                with open(self.profilepath, 'r') as file:
                    self.profile = yaml.load(file.read())

    def save_data(self):
        """Function: save.

        Save contents in megconfig.yml
        from settings.data and settings.project
        """
        if self.project:
            project_key = self._get_project_slug()
            self.project['dc_timestamp'] = os.path.getmtime(
                self.project['dc_path'])
            self.data['projects'][project_key] = self.project
            if not self.data['config']['current']:
                self.data['config']['current'] = project_key
        with open(self.datapath, 'w') as file:
            yaml.dump(self.data, file)

    def delete_project(self):
        """Function: delete_project.

        Delete current project:
        1. clear settings.project and settings.profile
        2. Remove project from megconfig.yml
        3. Remove all project user profiles from disk
        """
        ret = console.confirm(
            _("Do you really want to delete project"),
            default=False)
        if ret:
            project_key = self._get_project_slug()
            self.data['projects'].pop(project_key)
            remaining_projects = [proj for proj in self.data['projects']]
            if remaining_projects:
                new_proj = console.choose(remaining_projects)
            else:
                new_proj = None
            self.data['config']['current'] = new_proj
            with open(self.datapath, 'w') as file:
                yaml.dump(self.data, file)
            for file in glob.glob(
                    os.path.join(self.path, "{}*.yml".format(project_key))):
                os.remove(file)
            self.project = None
            self.profile = None

    def _validate_yaml(self, filepath):
        """Check if path is valid for yaml file.

        Parameter
        ---------
        filepath : path
            Absolute path for yaml file
        Return
        ------
        Boolean
            True if file can be read.
        """
        long_path = os.path.join(
            os.path.expanduser("~") if "~" in filepath else "",
            filepath.replace("~", ""),
            "docker-compose.yml"
            if "docker-compose" not in filepath else ""
        )
        print(_("Reading {}").format(long_path))
        try:
            with open(long_path, 'r') as file:
                yaml.load(file.read())
        except Exception as e:
            console.error(e)
            return False
        return True

    def _get_project_slug(self):
        """Slugify project name.

        - Convert " " to "_"
        - Lowercase all characters
        - Remove accents

        Return
        ------

        String: slugify version of string
        """
        return formatStr.slugify(self.project['name'])

    def _get_profile_path(self):
        config_file = "{}_{}.yml".format(
            self.data['config']['current'],
            getpass.getuser()
        )
        return os.path.join(self.path, config_file)



import memcache
import redis
import telnetlib
from megalus.core.li_tabulate import tabulate
from megalus.core.utils import console
from megalus.projects.config import profile


def run_service(service, key):

    console("Chaves do {}".format(service.upper()), style="section")

    data = profile.get_data()
    if not data:
        return False

    if service == "redis":
        key_list = get_all_redis_keys(data, key)
    else:
        key_list = get_all_memcached_keys(data, key)

    print((tabulate(
        key_list,
        headers=["Chave", "Valor"]
    )))


def get_all_redis_keys(data, key):
    conn = redis.StrictRedis(
        host=data['redis_host'],
        port=data['redis_port'],
        db=data['redis_db']
    )
    pattern = "*{}*".format(key) if key else "*"
    all_keys = set()
    for k in conn.scan_iter(pattern):
        if k not in all_keys:
            all_keys.add(k)
    all_keys = sorted(list(all_keys))

    key_list = []
    for k in all_keys:
        try:
            value = conn.get(k)
            if value:
                value = value.decode('utf-8')
            else:
                value = '--'
        except BaseException:
            chave = k.decode('utf-8')
            if '_kombu' in chave:
                value = '--'
            else:
                value = "Erro ao tentar recuperar o valor"
        key_list.append((k, value))

    return key_list


def get_all_memcached_keys(data, k):
    t = telnetlib.Telnet(
        data['memcached_host'],
        int(data['memcached_port']))
    t.write('stats items STAT items:0:number 0 END\n'.encode('ascii'))
    items = t.read_until('END'.encode('ascii')).split('\r\n'.encode('ascii'))
    keys = set()
    for item in items:
        item = item.decode('ascii')
        parts = item.split(':')
        if not len(parts) >= 3:
            continue
        slab = parts[1]
        t.write('stats cachedump {} 200000 ITEM '
                'views.decorators.cache.cache_header..cc7d9 '
                '[6 b; 1256056128 s] END\n'.format(
                    slab).encode('ascii'))
        cachelines = t.read_until(
            'END'.encode('ascii')).split(
            '\r\n'.encode('ascii'))
        for line in cachelines:
            line = line.decode('ascii')
            parts = line.split(' ')
            if not len(parts) >= 3:
                continue
            if not k or k in parts[1]:
                keys.add(parts[1])
    t.close()
    all_keys = sorted(list(keys))

    mc = memcache.Client(
        [
            '{}:{}'.format(data['memcached_host'], data['memcached_port'])
        ], debug=0
    )

    key_list = []
    for ky in all_keys:
        try:
            value = str(mc.get(ky))[:20] or '--'
        except BaseException:
            value = "Erro ao tentar recuperar o valor"
        key_list.append((ky, value))

    return key_list



"""[summary].

[description]
"""
import boto3
import os
import datetime
from botocore.exceptions import ClientError
from git import Repo
from time import strftime, sleep
from megalus.core.utils import console


class AWSManager():
    """AWS Manager Class."""

    def upload_to_s3(self, artifact):
        """
        Uploads an artifact to Amazon S3
        """
        self.version_label = strftime("%Y%m%d%H%M%S")
        self.last_commit = 'New build from Bitbucket'
        try:
            current_dir = os.getcwd()
            repo = Repo(current_dir)
            last_version = repo.tags[-1].name
            if last_version:
                self.version_label = "{}-{}".format(
                    last_version,
                    self.version_label
                )
            self.last_commit = repo.head.commit.message
        except BaseException:
            print('erro ao tentar encontrar a ultima tag')

        self.bucket_key = '{}/{}-bitbucket_builds.zip'.format(
            os.getenv('APPLICATION_NAME'),
            self.version_label
        )
        try:
            client = boto3.client('s3')
        except ClientError as err:
            print("Failed to create boto3 client.\n" + str(err))
            return False

        try:
            client.put_object(
                Body=open(artifact, 'rb'),
                Bucket=os.getenv('S3_BUCKET'),
                Key=self.bucket_key
            )
        except ClientError as err:
            print("Failed to upload artifact to S3.\n" + str(err))
            return False
        except IOError as err:
            print("Failed to access artifact.zip in this directory.\n" + str(err))
            return False

        return True

    def create_new_version(self):
        """
        Creates a new application version in AWS Elastic Beanstalk
        """
        try:
            client = boto3.client('elasticbeanstalk')
        except ClientError as err:
            print("Failed to create boto3 client.\n" + str(err))
            return False

        try:
            response = client.create_application_version(
                ApplicationName=os.getenv('APPLICATION_NAME'),
                VersionLabel=self.version_label,
                Description=self.last_commit,
                SourceBundle={
                    'S3Bucket': os.getenv('S3_BUCKET'),
                    'S3Key': self.bucket_key
                },
                Process=True
            )
        except ClientError as err:
            print("Failed to create application version.\n" + str(err))
            return False

        try:
            if response['ResponseMetadata']['HTTPStatusCode'] is 200:
                return True
            else:
                print(response)
                return False
        except (KeyError, TypeError) as err:
            print(str(err))
            return False

    def deploy_new_version(self):
        """
        Deploy a new version to AWS Elastic Beanstalk
        """
        try:
            client = boto3.client('elasticbeanstalk')
        except ClientError as err:
            print("Failed to create boto3 client.\n" + str(err))
            return False

        try:
            response = client.update_environment(
                ApplicationName=os.getenv('APPLICATION_NAME'),
                EnvironmentName=os.getenv('APPLICATION_ENVIRONMENT'),
                VersionLabel=self.version_label,
            )
        except ClientError as err:
            print("Failed to update environment.\n" + str(err))
            return False

        print(response)
        return True

    def check_deployment(self):
        finished = False
        result = 'pending'
        checks = 0
        first = True
        date_start = datetime.datetime.now()
        console("Checking Deployment", style="section")
        while not finished and checks < 10:
            check_api = False
            if first:
                check_api = True
                first = False
            else:
                date_now = datetime.datetime.now()
                elapsed_time = date_now - date_start
                period = int(elapsed_time.total_seconds() / 60)
                if period > checks:
                    check_api = True
                    checks = period
            if check_api:
                try:
                    client = boto3.client('elasticbeanstalk')
                    response = client.describe_environment_health(
                        EnvironmentName=os.getenv('APPLICATION_ENVIRONMENT'),
                        AttributeNames=['Status', 'Causes']
                    )
                    print("Check n. {}: Status: {} Reason: {}".format(
                        period + 1, response['Status'], response["Causes"][0]
                    ))
                    if response['Status'] == "Ready":
                        finished = True
                        result = 'finished'
                except ClientError as err:
                    print(
                        "Failed to check environment deployment.\n" +
                        str(err))
                    return 'pending'

        return result

    def deploy(self):
        " Your favorite wrapper's favorite wrapper "
        if not self.upload_to_s3('/tmp/artifact.zip'):
            return False
        if not self.create_new_version():
            return False
        # Wait for the new version to be consistent before deploying
        sleep(5)
        if not self.deploy_new_version():
            return False

        status = self.check_deployment()
        return status



"""[summary].

[description]
"""
import os
from megalus.core import utils
from megalus.projects.config import profile, run_update
from megalus.projects.setup import settings
from megalus.commands.docker import build


def run(no_confirm):
    """[summary].

    [description]

    Arguments:
        no_confirm {[type]} -- [description]

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False

    if no_confirm:
        resp = True
    else:
        resp = utils.confirma(
            "Este comando exclui todas as imagens\n"
            "e containers existentes na máquina,\n"
            "e inicia um novo Update/Build.\n"
            "\n\033[91mCertifique-se que você tenha um backup\n"
            "do banco de dados antes de rodar esse comando e\n"
            "que todas as alterações importantes"
            " estejam commitadas.\033[0m\n\n"
            "Deseja continuar")

    if resp:
        # Parar containers
        utils.stop_all(data)

        # Excluir containers e imagens
        utils.run_command(
            title="Excluir Containers do Docker",
            command_list=[
                {
                    'command': 'docker rm -f $(docker ps -a -q)',
                    'run_stdout': False
                }
            ]
        )
        utils.run_command(
            title="Excluir Imagens do Docker",
            command_list=[
                {
                    'command': 'docker rmi -f $(docker images -q)',
                    'run_stdout': False
                }
            ]
        )

        # Roda Update
        run_update(no_confirm=True, stable=True, staging=False)

        # Roda Build
        build.run(application=None)

        # Finaliza
        utils.notify(msg="Rebuild dos Containers finalizado")
        os.system('cls' if os.name == 'nt' else 'clear')
        print("O Rebuild foi concluído.")
        print("Antes de iniciar os containers, digite o(s) comando(s):")
        for dbdata in settings.LOCAL_DBS:
            print(
                "'cd {} && docker-compose up "
                "{}'".format(
                    data['docker_compose_path'],
                    dbdata['local_name']))
        print("para iniciar o Banco de dados pela primeira vez.")
        print("Em seguida use o comando 'meg run'.")
        return True



"""[summary].

[description]
"""
import os
from megalus.core import utils
from megalus.projects.config import profile


def run(application, dev):
    """[summary].

    [description]

    Arguments:
        application {[type]} -- [description]
        dev {[type]} -- [description]

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False

    container_id, name = utils.get_app(
        application=application,
        title="Rodar em modo Watch",
        data=data
    )
    if not name:
        return False

    dc_data = utils.get_compose_data(data)

    try:
        app_folder = dc_data['services'][name][
            'build']['context'].split('/')[-1]
    except BaseException:
        utils.console('Pasta não encontrada.', style="error")
        return False

    watch_path = os.path.join(
        data.project_path,
        app_folder,
        'frontend'
    )
    if dev:
        os.system(
            'cd {}/ && ./node_modules/.bin/webpack --config '
            'webpack.config.js\n'.format(
                watch_path)
        )

    os.system(
        'cd {}/ && ./node_modules/.bin/webpack --config '
        'webpack.config.js --watch\n'.format(
            watch_path)
    )



import platform
from megalus.core.utils import run_command
from buzio import formatStr

if platform.system().lower() == "windows":
    from win32api import *
    from win32gui import *
    import win32con
    import sys
    import os
    import struct
    import time


class WindowsBalloonTip:
    def __init__(self, title, msg):
        message_map = {
            win32con.WM_DESTROY: self.OnDestroy,
        }
        # Register the Window class.
        wc = WNDCLASS()
        hinst = wc.hInstance = GetModuleHandle(None)
        wc.lpszClassName = "PythonTaskbar"
        wc.lpfnWndProc = message_map  # could also specify a wndproc.
        classAtom = RegisterClass(wc)
        # Create the Window.
        style = win32con.WS_OVERLAPPED | win32con.WS_SYSMENU
        self.hwnd = CreateWindow(
            classAtom,
            "Taskbar",
            style,
            0,
            0,
            win32con.CW_USEDEFAULT,
            win32con.CW_USEDEFAULT,
            0,
            0,
            hinst,
            None)
        UpdateWindow(self.hwnd)
        iconPathName = os.path.abspath(
            os.path.join(sys.path[0], "balloontip.ico"))
        icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
        try:
            hicon = LoadImage(hinst, iconPathName,
                              win32con.IMAGE_ICON, 0, 0, icon_flags)
        except BaseException:
            hicon = LoadIcon(0, win32con.IDI_APPLICATION)
        flags = NIF_ICON | NIF_MESSAGE | NIF_TIP
        nid = (self.hwnd, 0, flags, win32con.WM_USER + 20, hicon, "tooltip")
        Shell_NotifyIcon(NIM_ADD, nid)
        Shell_NotifyIcon(NIM_MODIFY,
                         (self.hwnd, 0, NIF_INFO, win32con.WM_USER + 20,
                          hicon, "Balloon  tooltip", title, 200, msg))
        # self.show_balloon(title, msg)
        time.sleep(10)
        DestroyWindow(self.hwnd)

    def OnDestroy(self, hwnd, msg, wparam, lparam):
        nid = (self.hwnd, 0)
        Shell_NotifyIcon(NIM_DELETE, nid)
        PostQuitMessage(0)  # Terminate the app.


def notify(msg, title=None):
    if not title:
        title = "Megalus"
    if platform.system().lower() == 'linux':  # Linux
        run_command(
            task='notify-send -i gsd-xrandr {title} {msg}'.format(
                title='"{}"'.format(formatStr.unitext(title)),
                msg='"{}"'.format(formatStr.unitext(msg))
            )
        )
    elif platform.system().lower() == "windows" \
            and platform.version() == "10":  # Windows 10
        
        WindowsBalloonTip(msg, title)

    else:  # Mac
        run_command(
            task="osascript -e 'display notification "
            "{msg} with title {title}'".format(
                title='"{}"'.format(
                    formatStr.unitext(title)),
                msg='"{}"'.format(
                    formatStr.unitext(msg))
            )
        )



from textwrap import dedent
from megalus.core.utils import console
INICIAL = """
Este help mostra os comandos da ferramenta MEG-Tools.
Para apenas visualizar os comandos e suas opções, digite 'meg'.
Para ver a Ajuda de todos os comandos, digite 'meg help'.
Para ver a Ajuda de um comando específico digite 'meg help <comando>'.
"""

HELP_COMMANDS = [{'command': 'config',
                  'options': 'meg config [--clone-only]',
                  'description': '''
                  Use para configurar o ambiente de desenvolvimento local.
                  Use a opção `--clone-only` para clonar/baixar apenas
                  ''',
                  'examples': None,
                  'long_desc': None},
                 {'command': 'deploy',
                  'options': 'meg deploy',
                  'description': '''
                  Use para fazer o deploy de um ambiente local na Amazon.
                  É preciso rodar este comando na pasta-raiz de um dos repositorios
                  clonados pelo comando `meg config`
                  ''',
                  'examples': None,
                  'long_desc': None},
                 {'command': 'telnet',
                  'options': 'meg telnet <app> <port>',
                  'description': '''
                  Use para abrir a aplicação informada num TTY
                  que faz um telnet dentro do Docker no endereco 127.0.0.1
                  e a porta especificada.
                  Se não informar a aplicação a ferramenta irá perguntar
                  qual aplicação abrir, a partir dos containers em atividade.
                  Equivale ao comando "docker exec -ti _container_ telnet 127.0.0.1 _porta_"
                  ''',
                  'examples': 'meg telnet worker 6908',
                  'long_desc': None},
                 {'command': 'test',
                  'options': 'meg test [<app>] [--using=(django|nose|pytest)] [--rds] [-v]',
                  'description': '''
                  Use para rodar os testes unitários da aplicação.
                  Para usar este comandos as livrarias coverage, pydocstyle e pycodestyle devem
                  estar instaladas no container.
                  A ferramenta permite o uso das livrarias de teste:
                  - Django (pelo comando manage.py teste)
                  - Nose
                  - Pytest
                  Para isto basta informar qual utilizar com a opção "--using", em minúsculas. Estas
                  devem ser instaladas no container ou a ferramenta vai escolher a livraria
                  que estiver disponível.
                  Se nenhuma livraria estiver instalada no container ou nenhuma for selecionada,
                  será usado o Unittest padrão do Python.
                  Utilize as opções abaixo para configurar os testes:
                  * --using: Use uma das opções a seguir: nose, pytest, django
                  * --rds: Use o banco de dados na Amazon (utilizado pelo ambiente de Staging)
                    ao invés do banco de dados local, para rodar os testes
                  * -v use a opcao "verbose" para mostrar mais detalhes dos testes
                  ''',
                  'examples': '''
                  meg test painel (abre o Painel rodando a lib padrao e banco de dados local)
                  meg test site --using=django (abre o Site rodando django e banco de dados local)
                  ''',
                  'long_desc': None},
                 {'command': 'bash',
                  'options': 'meg bash <app>',
                  'description': '''
                  Use para abrir a aplicação informada num terminal bash (usuario root).
                  Se não informar a aplicação a ferramenta irá perguntar qual aplicação abrir,
                  a partir dos containers em atividade.

                  Equivale ao comando "docker exec -ti _container_ /bin/bash"
                  ''',
                  'examples': 'meg bash carrinho',
                  'long_desc': None},
                 {'command': 'run',
                  'options': 'meg run <app>',
                  'description': '''
                  Roda a aplicação informada e suas dependências ou todas as aplicações/containers
                  se não informar nenhuma aplicação.

                  Equivale ao comando "docker-compose stop && docker-compose up _aplicacao_"
                  ''',
                  'examples': '''
                  meg run (roda todos os containers)
                  meg run painel (roda o container do Painel e os containers dependentes
                  ''',
                  'long_desc': None},
                 {'command': 'build',
                  'options': 'meg build <app> [--no-cache]',
                  'description': '''
                  Use para fazer a build do container da aplicação selecionada.
                  Para atualizar somente a imagem base do Docker, use o comando 'meg build base'.
                  Se nenhuma aplicação for informada, será feito a build de todos os containers (mais
                  a imagem base). Opções:
                  * --no-cache: Durante a build, forçar o download de todas as livrarias/dependencias
                  Equivale ao comando "docker-compose stop && docker-compose build _aplicacao_"
                  ''',
                  'examples': '''
                  meg build (faz o build de todos os containers)
                  meg build loja (faz a build do container da Loja)
                  ''',
                  'long_desc': None},
                 {'command': 'update',
                  'options': 'meg update [--yes] [--production | --staging]',
                  'description': '''
                  Roda em cada repositório clonado pelo comando `meg config`
                  o comando "git remote update && git fetch && git pull --all",
                  desde que o repositorio esteja na branch prodution, staging,
                  beta, release ou master (repositórios que estejam em outros
                  branchs são ignorados).
                  Em adição ele clona novos repositórios que não estejam na pasta
                  do projeto informado.
                  Use a opção --yes ou -y para confirmar automaticamente as operações.
                  Use a opção --production para fazer o git checkout para a branch
                  mais estável do projeto (exemplo: production, master, etc) antes do update.
                  ''',
                  'examples': '''
                  meg update (atualiza todas as branchs atuais)
                  meg update --production (faz o checkout para a branch mais estável e atualiza)
                  meg update --staging (faz o checkout para staging/beta e atualiza os repositórios
                  ''',
                  'long_desc': None},
                 {'command': 'list',
                  'options': 'meg list [<libs>...]',
                  'description': '''
                  Lista as aplicações existentes no Docker-Compose, e informa as seguintes
                  informações:
                  * O nome da aplicação e se o container está rodando
                  * A branch em que o container está.
                  * As diferenças no branch, em relação ao VCS (commits a frente/atrás, etc...).
                  * A porta que está sendo exposta no container

                  Para listar outras livrarias, digite elas após o comando, separando com espaço.
                  ''',
                  'examples': '''
                  meg list (para a lista padrao)
                  meg list django flask gunicorn (Para listar as versões do Django,
                  Flask e Gunicorn que estão rodando dentro dos containers.)
                  ''',
                  'long_desc': None},
                 {'command': 'rebuild',
                  'options': 'meg rebuild',
                  'description': 'Este comando exclui todas as imagens e containers existentes na máquina.\nEm seguida atualiza os repositórios existentes e baixa os novos.\nApós isso, inicia a build de todos os containers.\nCertifique-se de ter feito um backup do seu banco de dados local, antes de iniciar esse comando.',
                  'examples': None,
                  'long_desc': None},
                 {'command': 'tunnel',
                  'options': 'meg tunnel <subdominio> <aplicacao>',
                  'description': 'Este comando cria um tunel reverso a partir de uma instância amazon (tunnel.awsli.com.br) para uma das aplicações rodando localmente.\nUse quando é necessário informar, em um serviço de terceiros, um endereço http ou https, ao invés de "localhost".\nO ngrok usa o <subdomínio> informado para criar a URL (subdominio.tunnel.awsli.com.br), tanto para http, quanto https e redireciona as chamadas para a porta da <aplicação> informada.\nEquivale ao comando "ngrok --subdomain=<subdominio informado> <porta informada>"',
                  'examples': 'meg tunnel retorno painel (Cria o tunel "http://retorno.tunnel.maisimovel.net" e redireciona as chamadas para a aplicação MaisPainel)\nmeg tunnel ret-facebook site (Cria o túnel "http://ret-facebook.maisimovel.net" e redireciona as requisições para o Site)',
                  'long_desc': None},
                 {'command': 'watch',
                  'options': 'meg watch <aplicacao>',
                  'description': 'Este comando indica ao Webpack para entrar em modo "watch".\nO webopack irá gerar os arquivos estáticos (html, css, png, etc...) para serem lidos pelo Django.\nCaso os arquivos sejam alterados o webpack atualizará automaticamente o arquivo comprimido.',
                  'examples': 'meg watch painel (comprime os arquivos estáticos do Painel)',
                  'long_desc': None},
                 {'command': 'ci',
                  'options': 'meg ci (major|minor|patch)',
                  'description': '''
                  Este comando inicia o Continuous Integration (CI).
                  O comando irá gerar um Pull Request para a branch 'master'
                  da aplicação onde o comando foi chamado.
                  É preciso estar na pasta raiz da aplicação.
                  A branch da aplicação deve ser a branch 'release'
                  O comando irá falhar se existirem modificações no git não commitadas.
                  O comando irá falhar se o último teste no Pipeline for falho.
                  O número da versão irá aumentar de acordo com a opção selecionada:
                  - major: x+1.0.0
                  - minor: 0.x+1.0
                  - patch: 0.0.x+1
                  Se nenhuma opção for selecionada, será usado a última tag criada.
                  ''',
                  'examples': 'meg ci patch (cria Pull Request e aumenta a versão x.x.Y+1)',
                  'long_desc': None},
                 {'command': 'resetdb',
                  'options': 'meg resetdb <application>',
                  'description': '''
                  Este comando irá apagar e recriar o banco de dados local da aplicação selecionada.
                  Certifique-se que qualquer dado importante tenha sido salvo antes de rodar este comando.
                  Após recriar o banco de dados, os comandos do Django, makemigrations, migrate e createdata
                  serão rodados no novo banco.''',
                  'examples': 'meg resetdb painel: recria o banco de dados do Painel',
                  'long_desc': None}]


def get_help(app=None):
    if app:
        command_help = [
            obj
            for obj in HELP_COMMANDS
            if obj.get('command') == app
        ]
        if command_help:
            show_help(command_help, show_long_desc=True)
            return True
        else:
            return True
    else:
        show_help(sorted(HELP_COMMANDS, key=lambda x: x['command']))
        return True


def show_help(help_list, show_long_desc=False):
    if not show_long_desc:
        print(dedent(INICIAL))
    for command in help_list:
        console(
            "Commando: ".format(
                command['command'].upper()),
            style="warning",
            use_prefix=False)
        if command['options']:
            console("Opções:", style="warning", use_prefix=False)
            print(command['options'])
        console("\nResumo:", style="warning", use_prefix=False)
        print(dedent(command['description']))
        if show_long_desc and command['long_desc']:
            print('\n{}'.format(dedent(command['long_desc'])))
        if command['examples']:
            console("\nExemplos:", style="warning", use_prefix=False)
            print(dedent(command['examples']))



# -*- coding: utf-8 -*-

import yaml
import os
from megalus.core.utils import get_app
from megalus.projects.config import profile


def run_ngrok(subdomain, app):
    # Busca configuração
    data = profile.get_data()
    if not data:
        return False

    # Busca docker-compose.yml
    dc_path = os.path.join(
        data['docker_compose_path'],
        'docker-compose.yml'
    )
    with open(dc_path, 'r') as file:
        dc_data = yaml.load(file)

    # Identifica aplicação
    name = ""
    container_id = None
    container_id, name = get_app(
        application=app,
        title="Rodar Túnel ngrok",
        data=data
    )
    if not container_id:
        return False

    # Busca a porta da aplicação
    try:
        services = dc_data['services']
        service_data = services[name]
        service_ports = service_data['ports']
        porta = service_ports[0].split(":")[0]
    except BaseException:
        porta = None

    if not porta:
        print("Nenhuma porta encontrada para a aplicação {}".format(name))
        return False

    os.system(
        "cd {} && ./ngrok -subdomain={} {}".format(
            data['docker_compose_path'], subdomain, porta)
    )

    return True



"""[summary].

[description]
"""
import os
import json
from colorama import Fore, Style
from megalus.core.li_tabulate import tabulate
from megalus.core import utils
from megalus.projects.config import profile


def run(application):
    """[summary].

    [description]

    Arguments:
        application {[type]} -- [description]

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False

    container_id, name = utils.get_app(
        application=application,
        title="Listagem NPM de Dependências",
        data=data
    )
    if not name:
        return False

    dc_data = utils.get_compose_data(data)

    try:
        if "_" in name:
            name = name.split("_")[1]
        app_folder = dc_data['services'][name][
            'build']['context'].split('/')[-1]
    except BaseException:
        utils.console('Pasta não encontrada.', style="error")
        return False

    watch_path = os.path.join(
        data.project_path,
        app_folder,
        'frontend'
    )

    ret_npm = utils.run_command(
        get_stdout=True,
        command_list=[
            {
                'command': 'cd {path} && npm ll -json'.format(path=watch_path),
                'run_stdout': False
            }
        ]
    )
    if not ret_npm:
        return False

    npm_json = json.loads(ret_npm)

    npm_dependencies = npm_json['dependencies']
    npm_depdev = npm_json['devDependencies']

    npm_list = []
    for dep in npm_dependencies:
        status = Fore.LIGHTGREEN_EX + "OK" + Style.RESET_ALL
        if npm_dependencies[dep]['extraneous']:
            status = Fore.LIGHTYELLOW_EX + 'Não incluído' + Style.RESET_ALL

        description = npm_dependencies[dep]['description']
        if len(description) > 60:
            description = description[:56] + ' ...'

        if dep in npm_depdev:
            dep_type = Fore.LIGHTBLACK_EX + "Dev" + Style.RESET_ALL
        else:
            dep_type = "Prod"

        npm_list.append(
            (npm_dependencies[dep]['name'],
             npm_dependencies[dep]['version'],
             dep_type,
             status,
             description)
        )

    final_list = sorted(npm_list, key=lambda x: x[0])

    print(tabulate(
        final_list,
        headers=['Nome', "Versão", "Tipo", "Status", "Descrição"]
    )
    )



"""[summary].

[description]

Variables:
    profile {[type]} -- [description]
"""
import os
from git import Repo
from os.path import expanduser
from megalus.core.messages import notify
from megalus.core import utils
from megalus.core.li_tabulate import tabulate
from megalus.projects.setup import settings


class Config():
    def __init__(self, filename=None, *args, **kwargs):
        self.data = settings.CONFIG_DICT
        proj_name = getattr(settings, 'ENV_NAME', None)
        if proj_name:
            self.project_path = os.environ.get(proj_name)
        else:
            self.project_path = None
        if filename:
            self.filename = filename
        else:
            self.filename = getattr(settings, 'CONFIG_FILE', None)

    def check_config(self, verbose=False):
        ret = True
        if not self.project_path:
            if verbose:
                utils.console(
                    "Defina a variável {} antes de continuar.".format(
                        settings.ENV_NAME), style="error")
            ret = False
        for envvar in settings.ENV_LIST:
            if not os.environ.get(envvar):
                if verbose:
                    utils.console(
                        "Defina a variável {} "
                        "antes de continuar.".format(envvar), style="error")
                ret = False
        path = self.get_filename()
        if not os.path.exists(path):
            if verbose:
                utils.console(
                    "O arquivo {} não foi encontrado".format(
                        path), style="error")
            ret = False
        else:
            loaded_dict = {}
            with open(path, 'r') as file:
                for line in file:
                    if "=" in line:
                        key = line.split("=")[0].lower()
                        value = line.split("=")[1].rstrip()
                        loaded_dict[key] = value

            for key in self.data:
                if not loaded_dict.get(key):
                    if verbose:
                        utils.print_error(
                            "O arquivo de configuração não "
                            "tem a chave {} informada.".format(key))
                    ret = False
        if ret and verbose:
            utils.print_success("Configuração OK.")
        return ret

    def show_config(self):
        utils.console("Configuração Atual:", style="warning", use_prefix=False)
        utils.console(
            "Arquivo de configuração: {}".format(
                self.get_filename()), style="warning", use_prefix=False)
        utils.console(
            "Caminho do Projeto ({}): {}\n".format(
                settings.ENV_NAME,
                self.project_path), style="warning", use_prefix=False)
        data = self.get_data(force=True)
        config_list = [
            (obj, data[obj])
            for obj in data
        ]
        print(tabulate(
            config_list,
            headers=["Opção", "Valor"]
        ) + "\n")

    def get_filename(self):
        filepath = os.path.join(expanduser("~"), ".{}".format(self.filename))
        return filepath

    def get_data(self, verbose=False, force=False):
        if self.check_config(verbose=verbose) or force:
            self.data.project_path = self.project_path
            with open(self.get_filename(), 'r') as file:
                for line in file:
                    if "=" in line:
                        key = line.split("=")[0].lower()
                        value = line.split("=")[1].rstrip()
                        self.data[key] = value
            return self.data
        else:
            return False

    def clone_repositories(self, force=False):

        if force:
            resp = True
        else:
            resp = utils.confirma("\nDeseja clonar os Repositórios")
        if resp:
            if not os.path.exists(self.project_path):
                os.makedirs(self.project_path)
            utils.run_command(
                title="Clonando Repositorios",
                command_list=[
                    {
                        'command': "git config --global "
                        "credential.helper 'cache --timeout=3600'",
                        'run_stdout': False}])

            # Baixa APPLICATIONS
            baixa_repositorios(self.project_path)

    def set_data(self, force=False):
        utils.console("Configuração", style="section")
        if force:
            resp = True
        else:
            resp = utils.confirma("Deseja configurar as chaves")
        if resp:
            with open(self.get_filename(), 'w') as file:
                for key in self.data:
                    if key == "docker_compose_path" and not self.data.get(key):
                        continue
                    if self.data.get(key):
                        ask = "Informe {} [{}]: ".format(
                            key.upper(), self.data.get(key))
                    else:
                        ask = "Informe {}: ".format(key.upper())

                    resposta_ok = False
                    while not resposta_ok:
                        try:
                            value = str(input(ask))
                            if not value and self.data[key]:
                                file.write(
                                    "{}={}\n".format(
                                        key.upper(), self.data[key]))
                                resposta_ok = True
                            if value:
                                self.data[key] = value
                                file.write(
                                    "{}={}\n".format(
                                        key.upper(), value))
                                resposta_ok = True
                        except KeyboardInterrupt:
                            print("\nOperação interrompida")
                            return False
                        except BaseException:
                            print('erro')
                            pass
            # Grava arquivo de credenciais da Amazon
            if settings.USE_AWS:
                resp = utils.confirma(
                    "Deseja gravar o arquivo com as credenciais da Amazon?")
                if resp:
                    aws_folder = os.path.join(expanduser("~"), ".aws")
                    if not os.path.exists(aws_folder):
                        os.makedirs(aws_folder)
                    with open(os.path.join(aws_folder, "config"), 'w') as file:
                        file.write("[config]\n")
                        file.write(
                            'region = {}\n'.format(
                                self.data['aws_region']))

                    with open(os.path.join(
                            aws_folder, "credentials"), 'w') as file:
                        file.write('[default]\n')
                        file.write(
                            'aws_access_key_id = {}\n'.format(
                                self.data['aws_key']))
                        file.write(
                            'aws_secret_access_key = {}\n'.format(
                                self.data['aws_secret']))

        # Confirma o caminho do docker-compose.yml
        if not self.data.get('docker_compose_path'):
            ret = utils.run_command(
                title="Localizando arquivo docker-compose.yml",
                get_stdout=True,
                command_list=[
                    {
                        'command': "locate docker-compose.yml",
                        'run_stdout': False
                    }
                ]
            )
            if ret:
                paths_found = ret.split('\n')
                if paths_found[-1] == '':
                    paths_found.pop(-1)
                if len(paths_found) == 1:
                    self.data['docker_compose_path'] = paths_found[
                        0].replace('docker-compose.yml', '')
                elif paths_found:
                    print(
                        "Informe a localização do arquivo"
                        " 'docker-compose.yml' do Projeto")
                    print("(A localização padrão é: '{}/{}')\n".format(
                        self.project_path,
                        settings.DOCKER_REPO_NAME
                    ))
                    print("Os caminhos encontrados foram:")
                    for num, path in enumerate(paths_found):
                        print("{}. {}".format(num + 1, path))
                    resposta_ok = False
                    print("\n")
                    while not resposta_ok:
                        try:
                            rep = input(
                                "Selecione o "
                                "caminho: (1-{}): ".format(num + 1))
                            if rep and int(rep) in range(1, num + 1):
                                resposta_ok = True
                        except KeyboardInterrupt:
                            print("Operação interrompida\n")
                            return False
                        except BaseException:
                            pass
                    self.data['docker_compose_path'] = paths_found[
                        int(rep) - 1].replace('docker-compose.yml', '')
            else:
                resposta_ok = False
                while not resposta_ok:
                    try:
                        default_path = os.path.join(
                            self.project_path, settings.DOCKER_REPO_NAME)
                        rep = input(
                            "Informe o caminho do arquivo "
                            "docker-compose.yml [{}]: ".format(default_path))
                        if rep == "":
                            rep = default_path
                        if os.path.exists(
                            os.path.join(
                                rep,
                                "docker-compose.yml")):
                            resposta_ok = True
                            self.data['docker_compose_path'] = rep
                    except KeyboardInterrupt:
                        print("Operação interrompida\n")
                        return False
                    except BaseException:
                        pass

            if self.data.get('docker_compose_path'):
                print('Arquivo encontrado!')
                with open(self.get_filename(), 'a') as file:
                    file.write(
                        "{}={}\n".format(
                            "DOCKER_COMPOSE_PATH",
                            self.data.get('docker_compose_path')
                        ))

        notify(msg="Configuração finalizada.")
        return True


profile = Config()


def run_update(no_confirm, stable, staging):
    data = profile.get_data()

    if not data:
        return False

    utils.console("Atualizar Repositórios", style="section")

    if no_confirm:
        resp = True
    else:
        resp = utils.confirma(
            "Este comando atualiza todos os Repositórios\n"
            "que estejam nas branchs 'production', 'staging'\n"
            "'release', 'beta' ou 'master'.\n"
            "Além disso, baixa novos repositórios que ainda\n"
            "não estejam na pasta do projeto.\n"
            "Deseja continuar")

    if resp:
        # Iterar todos os repositórios
        # Checar em que branch está
        # Se tiver em production ou master, dar o git pull
        for app in settings.APPLICATIONS:
            app_name = app[0]
            stable_branch = app[1][-1]
            caminho = os.path.join(data.project_path, app_name)
            if os.path.exists(caminho):
                repo = Repo(
                    os.path.join(data.project_path, app_name)
                )
                # 1. Checa se o remote existe e é válido
                remote_url_list = repo.remotes
                origin = [
                    obj
                    for obj in remote_url_list
                    if obj.name == 'origin'
                ]
                if origin:
                    origin = origin[0]
                remote_url = "{}{}.git".format(
                    settings.VCS_BASE_URL,
                    app_name.lower())
                if origin:
                    if origin.url != remote_url:
                        if no_confirm:
                            resp = True
                        else:
                            resp = utils.confirma(
                                "O repositório '{}' está com o endereço\n"
                                "remoto: {}.\nDeseja trocar".format(
                                    app_name, origin.url))
                        if resp:
                            origin.set_url(
                                new_url=remote_url
                            )
                else:
                    repo.create_remote(name="origin", url=remote_url)

                # 2. Se tiver a opção 'stable'
                # mudar para a branch mais estável
                if stable:
                    os.chdir(caminho)
                    utils.run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git checkout {branch}'.format(
                                    branch=stable_branch
                                ),
                                'run_stdout': False
                            }
                        ]
                    )

                # Checa Staging
                if staging:
                    stage_branch = None
                    if 'staging' in app[1]:
                        stage_branch = 'staging'
                    if 'beta' in app[1]:
                        stage_branch = 'beta'

                    if not stage_branch:
                        stage_branch = stable_branch

                    os.chdir(caminho)
                    utils.run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git checkout {branch}'.format(
                                    branch=stage_branch
                                ),
                                'run_stdout': False
                            }
                        ]
                    )

                # 3. Checa se o repositorio é production/master
                # e faz o update
                branch = repo.active_branch.name
                if branch in [
                    'production',
                    'staging',
                    'beta',
                    'release',
                        'master']:
                    utils.console(
                        "Atualizando {}/{}".format(app_name, branch),
                        style="info",
                        use_prefix=False
                    )
                    os.chdir(caminho)
                    utils.run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git remote update '
                                '&& git fetch && git pull --all',
                                'run_stdout': False
                            }
                        ]
                    )
                else:
                    utils.print_warning("Repositório '{}' "
                                        "ignorado. Branch: {}".format(
                                            app_name, branch))
        # Baixa os repositorios faltantes
        baixa_repositorios(data.project_path)
        notify(msg="Update dos projetos finalizado.")


def baixa_repositorios(path):
    for app, branch_list in settings.APPLICATIONS:
        if os.path.exists(os.path.join(path, app)):
            continue
        utils.print_warning("Baixando '{}'".format(app))
        first_branch = True
        for branch in branch_list:
            github_url = "{}{}.git".format(
                settings.VCS_BASE_URL,
                app.lower())
            if first_branch:
                ret = utils.run_command(
                    title=None,
                    command_list=[
                        {
                            'command': 'git clone -b'
                            ' {branch} {url} "{dir}"'.format(
                                branch=branch,
                                url=github_url,
                                dir=os.path.join(
                                    path,
                                    app)),
                            'run_stdout': False}])
                first_branch = False
            else:
                if ret:
                    os.chdir(os.path.join(path, app))
                    utils.run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git checkout'
                                ' -b {branch} remotes/origin/{branch}'.format(
                                    branch=branch
                                ),
                                'run_stdout': False
                            }
                        ]
                    )



import os
import re
from colorama import Back
from git import Repo
from megalus.core import utils
from megalus.core.li_tabulate import tabulate
from megalus.projects.config import profile
from megalus.projects.setup import settings


def run(libs=[]):
    data = profile.get_data()
    if not data:
        return False

    dc_data = utils.get_compose_data(data)

    utils.console("Lendo Aplicações Docker", style="section")

    services_list = dc_data['services']

    table_data = []
    total = len(services_list)
    i = 0
    blank = max([
        len(obj)
        for obj in services_list
    ])
    for service in sorted(services_list):
        # Barra de progresso
        i += 1
        utils.progress_bar(i, total, suffix=service.ljust(blank))
        # 1. Checa se o container está rodando
        docker_ret = utils.run_command(
            command_list=[
                {
                    'command': "docker ps | grep {service}".format(
                        service=service),
                    'run_stdout': False
                }
            ],
            get_stdout=True,
            title=False
        )
        if docker_ret:
            container = docker_ret[:12]
        else:
            container = None

        rodando = utils.console(
            "Sim" if container else "Não",
            style="success" if container else "error",
            use_prefix=False,
            format_only=True
        )

        # 2. Checa a branch
        try:
            caminho_dict = services_list.get(service)
            path = caminho_dict.get('build').get('context')
            caminho_path = os.path.join(
                data.project_path,
                *path.split("/")[1:]
            )
            repo = Repo(caminho_path)
            branch = repo.active_branch.name
        except BaseException:
            branch = "--"
            caminho_path = None

        # 3. Checa status do Git
        if branch != "--":
            os.chdir(caminho_path)
            status_ret = utils.run_command(
                command_list=[
                    {
                        'command': 'git status -bs --porcelain',
                        'run_stdout': False
                    }
                ],
                get_stdout=True,
                title=None
            )
            if status_ret.count('\n') > 1 or "[" in status_ret:
                branch = utils.console(
                    branch,
                    style="warning",
                    use_prefix=False,
                    format_only=True)
                ahead = re.search("ahead (\d+)", status_ret)
                behind = re.search("behind (\d+)", status_ret)

                ahead_text = utils.console(
                    "[+{}]".format(ahead.group(1)),
                    style="info",
                    use_prefix=False,
                    format_only=True
                ) if ahead else ""

                behind_text = utils.console(
                    "[-{}]".format(behind.group(1)),
                    style="error",
                    use_prefix=False,
                    format_only=True
                ) if behind else ""

                branch = utils.console(
                    " {}{}{} ".format(branch, ahead_text, behind_text),
                    style="git"
                )

        # 4. Checa versão das livrarias
        lib_list = []
        pip_ret = None
        if caminho_path:
            for lib in settings.LIBRARIES + libs:
                if container:
                    pip_ret = utils.run_command(
                        command_list=[
                            {
                                'command': 'docker exec -ti '
                                '{container} pip freeze | grep -i '
                                '{library}== | tail -1'.format(
                                    container=container,
                                    library=lib),
                                'run_stdout': False}],
                        get_stdout=True,
                        title=None)
                elif branch != "--":
                    try:
                        pip_ret = utils.run_command(
                            command_list=[
                                {
                                    'command': 'cd {path} && '
                                    'docker-compose run {service} '
                                    'pip freeze | grep -i {library}== '
                                    '| tail -1'.format(
                                        path=data['docker_compose_path'],
                                        service=service,
                                        library=lib),
                                    'run_stdout':False}],
                            get_stdout=True,
                            title=None)
                    except BaseException:
                        pip_ret = None
                if pip_ret:
                    lib_list.append(pip_ret.split("==")[1])
                else:
                    lib_list.append("--" if "SIM" in rodando else "")

        # 5. Pega a Porta
        try:
            porta = dc_data['services'][service]['ports'][0].split(":")[0]
        except BaseException:
            porta = "--"

        table_data.append([service, branch, rodando, porta] + lib_list)

    # Exclui containers extra
    os.system(
        "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
    )
    os.system('cls' if os.name == 'nt' else 'clear')
    utils.console("Listar Aplicações Docker", style="section")
    print(
        tabulate(
            table_data,
            headers=[
                "Aplicação",
                "Branch",
                "Rodando",
                "Porta"] +
            settings.LIBRARIES +
            libs))



"""Ferramenta Megalus.
Para mais detalhes digite 'meg help'

Usage:
    meg bash        [<app>]
    meg build       [<app>] [--no-cache]
    meg config      [--clone-only]
    meg debug       [<app>]
    meg deploy
    meg configproject
    meg help        [<option>]
    meg list        [<libs>...]
    meg npm list    <app>
    meg rebuild     [-y | --yes]
    meg release     [(major|minor|patch)]
    meg release eb
    meg resetdb     [<app>]
    meg run         [<app>] [<command> ...]
    meg service     (redis|memcached) [<key>]
    meg telnet      [<app>] (<port>)
    meg test        [<app>] [--using=(django|nose|pytest)] [--rds] [-v]
    meg tunnel      [<subdomain>] [<app>]
    meg update      [-y | --yes] [--production | --staging]
    meg watch       [<app>] [--dev]

Options:
    --help          Mostra esta tela
    --django        Roda o teste unitario do Django. (Padrao: Unittest.)
    --rds           Nos testes usar o RDS da Amazon
    --no-cache      Na build nao utilizar o cache
    <command>       Rode um comando para o run do container
    -y --yes        Confirma automaticamente
    <libs>          Mostrar a versão das livrarias solicitadas
    --production    Muda para a branch mais estável (ex. 'production')
    --staging       Altera as branchs para staging/beta durante o update
    <subdomain>     O subdominio para o tunel reverso, via ngrok
    <app>           Aplicacao que sera alvo do comando
    -v              Verbose

"""
import sys
from docopt import docopt
from megalus import __version__
from megalus.commands import aws
from megalus.commands import docker
from megalus.commands import npm
from megalus.core.help import get_help
from megalus.core.release import make_pull_request, start_deploy
from megalus.core.services import run_service
from megalus.core.tunnel import run_ngrok
from megalus.core.utils import (
    confirma, run_command, console
)
from megalus.projects.config import profile, run_update
from megalus.projects.setup import settings
from megalus.version import show_version_warning


def check_vpn():
    """Checa se a VPN está ativa."""
    if settings.CHECK_VPN:
        ret_tun = run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': 'ifconfig | grep tun',
                    'run_stdout': False
                }
            ]
        )

        if not ret_tun:
            print("\n{}{}ERRO:{} VPN não encontrada.".format(
                bcolors.BOLD, bcolors.FAIL, bcolors.ENDC))
            print("Por favor, ative a VPN e tente novamente.")
            return False

    return True


def main(arguments):
    """Faz o Parse dos Comandos."""

    if not arguments['help']:
        console("Para ajuda digite: meg help", style="grey", use_prefix=False)

    #
    # CONFIG
    #
    if arguments['config'] is True and check_vpn():
        clone_only = arguments['--clone-only']

        profile.show_config()
        ret = profile.check_config(verbose=True)

        resposta = confirma("Deseja rodar a configuração?")
        if resposta:
            profile.set_data()

        return True
    #
    # DEPLOY
    #
    if arguments['deploy'] is True and check_vpn():
        ret = aws.deploy.run()
        return ret
    #
    # DEBUG
    #
    if arguments['debug'] is True:
        ret = docker.debug.run(
            application=arguments['<app>']
        )
        return ret
    #
    # TELNET
    #
    if arguments['telnet'] is True:
        ret = docker.telnet.run(
            application=arguments['<app>'],
            port=arguments['<port>']
        )
        return ret
    #
    # BASH
    #
    if arguments['bash'] is True:
        ret = docker.bash.run(
            application=arguments['<app>']
        )
        return ret
    #
    # TEST
    #
    if arguments['test'] is True:
        ret = docker.test.run(
            application=arguments['<app>'],
            using=arguments['--using'],
            rds=arguments['--rds'],
            verbose=arguments['-v']
        )
        return ret
    #
    # RUN APP
    #
    if arguments['run'] is True:
        ret = docker.run.run(
            application=arguments['<app>'],
            action='up' if not arguments['<command>'] else 'exec',
            arg=arguments['<command>']
        )
        return ret
    #
    # BUILD APP
    #
    if arguments['build'] is True:
        ret = docker.build.run(
            application=arguments['<app>']
        )
        return ret
    #
    # UPDATE
    #
    if arguments['update'] is True:
        ret = run_update(
            no_confirm=arguments['--yes'],
            stable=arguments['--production'],
            staging=arguments['--staging'])
        return ret
    #
    # REBUILD
    #
    if arguments['rebuild'] is True:
        ret = docker.rebuild.run(no_confirm=arguments['--yes'])
        return ret
    #
    # HELP
    #
    if arguments['help'] is True:
        ret = get_help(app=arguments['<option>'])
        return ret
    #
    # LIST
    #
    if arguments['list'] and not arguments['npm']:
        ret = docker.lists.run(libs=arguments['<libs>'])
        return ret
    #
    # TUNNEL
    #
    if arguments['tunnel'] is True and check_vpn():
        ret = run_ngrok(
            subdomain=arguments['<subdomain>'],
            app=arguments['<app>']
        )
        return ret
    #
    # SERVICE
    #
    if arguments['service'] is True:
        if arguments['redis']:
            service = 'redis'
        else:
            service = 'memcached'
        ret = run_service(
            service=service,
            key=arguments['<key>']
        )
        return ret
    #
    # WATCH
    #
    if arguments['watch'] is True:
        ret = npm.watch.run(
            application=arguments['<app>'],
            dev=arguments['--dev'])
        return ret
    #
    # NPM LIST
    #
    if arguments['npm'] and arguments['list']:
        ret = npm.list.run(application=arguments['<app>'])
        return ret
    #
    # RELEASE
    #
    if arguments['release'] and not arguments['eb']:
        if arguments['major']:
            release = 'major'
        elif arguments['minor']:
            release = 'minor'
        elif arguments['patch']:
            release = 'patch'
        else:
            release = "same"
        ret = make_pull_request(release=release)
        return ret
    #
    # RELEASE EB
    #
    if arguments['release'] and arguments['eb']:
        ret = start_deploy()
        return ret
    #
    # RESET DB
    #
    if arguments['resetdb'] is True:
        ret = docker.resetdb.run(application=arguments['<app>'])
        return ret
    #
    # SHOW PROJECT
    #
    if arguments['configproject'] is True:
        ret = settings.show_project()
        return ret


def start():

    arguments = docopt(__doc__, version=__version__)

    if not settings.loaded:
        ret = confirma(
            "A configuração do Projeto {} não foi encontrada.\n"
            "Deseja criá-la agora".format(settings.project))
        if ret:
            retorno = settings.create_project()
        else:
            sys.exit(1)
    else:
        console(
            "Megalus v.{}".format(__version__),
            style="main_title",
            use_prefix=False)
        console(
            "Projeto: {}".format(
                settings.project),
            style="info",
            use_prefix=False)
        show_version_warning()
        retorno = main(arguments)
    if not retorno:
        print('\n')
    else:
        print("\nOperacao finalizada.\n")

    sys.exit(retorno)


if __name__ == "__main__":
    start()



"""
Settings para o MEG-Tools

Chaves do JSON:

CONFIG_FILE = Nome do arquivo de configuração (ex: "meg-config")
ENV_NAME = Variável de ambiente com o path do projeto (ex: "MEGALUS_PATH")
ENV_LIST = Lista com as variáveis de ambiente obrigatórias no projeto.
LOCAL_DBS = Lista com a configuração de cada banco de dados do projeto:
ex: [
    {
        'local_name': "service.postgres.local",
        'desc': "Banco de dados do Painel",
        'local_port': 5432,
        'user': 'nome_do_usuario',
        'name': 'nome_do_banco',
        'admin': "nome do container docker usado para enviar os comandos (ex.:frontend.painel.local)",
        'stage_name': "",
        'stage_port': 5432
    }
]
CHECK_VPN = Usa VPN para acessar AWS?
TEST_PRIORITY = Lista com os programas para rodar testes, na ordem de prioridade.
Ex.: [
    'nose',
    'pytest'
]
APPLICATIONS = Lista de Aplicações do Projeto.
A ordem das branchs é da mais volátil até a mais estável.
Ex.: [
    ['app1', ['master']],
    ['app2', ['develop', 'master']]
]
LIBRARIES = Lista de livrarias para mostrar no comando meg list:
Ex.: [
    'Django'
]
MINIFY_BEFORE = Aplicações que minificam arquivos durante o deploy
ex.: [
    'app1'
]
SYNC_S3 = Aplicações que sincronizam arquivos no S3 durante o deploy
Ex.: [
    'app1'
]
BASEDIRSTATIC = Caminho da pasta base para Compressão de arquivos estáticos nos aplicativos acima
Ex.: [
    "dir1", "dir2", static"
]
JSSOURCES = Caminho relativo a base dos arquivos JS para minificar num unico arquivo.
Todos os arquivos são comprimidos no arquivo "all.min.js"
Ex.: [
    ['folder3', 'index.js'],
    ['folder4', 'app.js']
]
CSSSOURCES = Caminho relativo a base dos arquivos CSS para minificar num unico arquivo.
Todos os arquivos são comprimidos no arquivo "all.min.css"
Ex.: [
    ['folder3', 'index.css'],
    ['folder4', 'app.css']
]
JSALONE = Arquivos JS para minificar individualmente

Ex.: [
    ["js", "index.js"]
]
#
CSSALONE =  Arquivos CSS para minificar individualmente
Exemplo: index.css será salvo como index.min.css
Ex.: [
    ["css", "index.css"]
]
DOCKER_REPO_NAME = Nome do projeto com o Docker-Compose.yaml (ex.: "megdocker")
DOCKER_BASE_IMAGE_REPO = Imagem usada como base para a build das restantes (ex.: "megbase")
DOCKERFILE_DEPLOY = Nome do arquivo Dockerfile para deploy. Ex: "Dockerfile_deploy"
DOCKERFILE_DEVELOPMENT = Nome do arquivo Dockerfile para develop. Ex.:"Dockerfile_dev"
VCS_NAME = Nome do VCS. Ex: "Bitbucket"
VCS_BASE_URL = URL base do VCS: Ex.: "https://bitbucket.org/NOME/"
VCS_BASE_API_URL = URL base para API. Ex.: 'https://api.bitbucket.org/'
REPOSITORY_API_URL = '2.0/repositories/NOME/{repo}/commit/{commit}/statuses'
PULL_REQUEST_API_URL = '2.0/repositories/NOME/{repo}/pullrequests/'
USE_AWS = Projeto hospedado na Amazon?
S3_SYNC_CMD = Comando para sincronizar o S3. Ex.: "aws s3 sync static/ s3://dominio.cdn/{branch}/static/ --acl public-read"
USE_ECR = Projeto hospeda as imagens Docker no ECR Amazon?
ECR_NAME = Faz o De/Para do nome do projeto para o nome do container no ECS
Ex.: {
    'app1': 'prod.app'
}
USE_REDIS = Usa Redis?
USE_MEMCACHED = Usa Memcached?
USE_SLACK = Usa Slack?
TEST_CHANNEL = Canal para testes no Slack. Ex.: "#teste_automacao"
USE_DATADOG = Usa Datadog?
USE_GRAFANA = Usa Grafana?
GRAFANA_MSG_URL = URL para enviar eventos ao Grafana. Ex: "http://dominio:porta/write?db=msgs"
"""
import os
import json
import yaml
from os.path import expanduser
from collections import OrderedDict
from megalus.core import utils
from megalus.core.li_tabulate import tabulate


class Setting():
    """Settings object.

    This objects has the PROJECT properties loaded from
    a json file and the USER properties define
    on the CONFIG_DICT dictionary.
    """

    def __init__(self):
        """Init definition."""
        self.project = None
        basepath = expanduser("~")
        self.path = os.path.join(basepath, ".megalus")
        self.loaded = False

    def show_projects(self):
        """[summary].

        [description]

        Returns:
            [type] -- [description]
        """
        all_projects = [
            filename.replace(".json", "").replace("_", " ").title()
            for root, dirs, file in os.walk(self.path)
            for filename in file
            if filename.endswith(".json")
        ]
        if all_projects:
            utils.console("Projetos disponíveis", style="section")
            for i, app in enumerate(all_projects):
                print("{}. {}".format(i + 1, app))
            print("\n")

    def check_file(self):
        """[summary].

        [description]

        Returns
        -------
            [type] -- [description]

        """
        filename = os.path.join(self.path, "{}.json".format(
            self.project.replace(" ", "_").lower()
        ))
        return os.path.isfile(filename)

    def load(self):
        """[summary].

        [description]
        """
        self.get_current_project()
        if not self.project:
            self.show_projects()
            self.loaded = False
        else:
            if not self.check_file():
                self.loaded = False
            else:
                filename = os.path.join(self.path, "{}.json".format(
                    self.project.replace(" ", "_").lower()
                ))
                with open(filename, "r") as file:
                    config_dict = json.loads(file.read())
                    for key in config_dict:
                        setattr(self, key, config_dict[key])
                self.loaded = True

    def show_project_config(self):
        """[summary].

        [description]

        Returns:
            bool -- [description]
        """
        self.read()
        if not self.loaded:
            return False

        os.system('cls' if os.name == 'nt' else 'clear')
        utils.console(
            "Configuração do projeto {}".format(
                self.project),
            style="section")
        table_data = []
        for key in dir(settings):
            if "__" not in key and key == key.upper() and key != "CONFIG_DICT":
                data = getattr(settings, key)
                if isinstance(data, list):
                    if key == "APPLICATIONS":
                        for i, item in enumerate(data):
                            table_data.append([
                                utils.humanize(key) if i == 0 else "",
                                item[0],
                                utils.humanize(item[1])
                            ])
                    elif key == "LOCAL_DBS":
                        for item in data:
                            for i, config in enumerate(item):
                                table_data.append([
                                    utils.humanize(key) if i == 0 else "",
                                    item['local_name'] if i == 0 else "",
                                    "{}: {}".format(
                                        utils.humanize(config), item[config])
                                ])
                    else:
                        for i, item in enumerate(data):
                            table_data.append([
                                utils.humanize(key) if i == 0 else "",
                                utils.humanize(item)
                            ])
                else:
                    table_data.append(
                        [utils.humanize(key), data]
                    )
        print(tabulate(
            table_data,
            headers=["Variável", "Valor", "Detalhe"]
        ))
        resp = utils.confirma("Deseja atualizar os dados")

        return True

    def create_project(self):
        
        utils.console("Nova configuração para Projeto\n"
            "Projeto selecionado: {}".format(self.project),
            style="main_title",
            use_prefix=False
        )
        dc_path = input("Informe o caminho completo do arquivo docker-compose.yml: ")
        try:
            with open(dc_path, 'r') as file:
                dc_data = yaml.load(file)
        except:
            pass


    def get_current_project(self):
        if not os.path.isdir(self.path):
            os.makedirs(self.path)
        setup_path = os.path.join(self.path, '.setup')
        if os.path.isfile(setup_path):
            with open(setup_path, 'r') as file:
                setup_data = json.loads(file.read())
        else:
            setup_data = {
                'project_name': None
            }
            with open(setup_path, 'w') as file:
                json.dump(setup_data, file)
        self.project = setup_data['project_name']


settings = Setting()
settings.load()

settings.CONFIG_DICT = OrderedDict.fromkeys(
    [
        "docker_compose_path",
        "use_for_tests",
        "vcs_username",
        'vcs_password'
    ]
)
settings.CONFIG_DICT['use_for_tests'] = 'unittest'

if getattr(settings, 'USE_AWS', None):
    settings.CONFIG_DICT['aws_key'] = None
    settings.CONFIG_DICT['aws_secret'] = None
    settings.CONFIG_DICT['aws_region'] = None
    settings.CONFIG_DICT['aws_account'] = None

if getattr(settings, 'USE_SLACK', None):
    settings.CONFIG_DICT['slack_url'] = None
    settings.CONFIG_DICT['slack_channel'] = None
    settings.CONFIG_DICT['slack_icon'] = None
    settings.CONFIG_DICT['slack_user'] = None

if getattr(settings, 'USE_DATADOG', None):
    settings.CONFIG_DICT['datadog_api_key'] = None
    settings.CONFIG_DICT['datadog_app_key'] = None

if getattr(settings, 'USE_REDIS', None):
    settings.CONFIG_DICT['redis_host'] = None
    settings.CONFIG_DICT['redis_port'] = None
    settings.CONFIG_DICT['redis_db'] = None

if getattr(settings, 'USE_MEMCACHED', None):
    settings.CONFIG_DICT['memcached_host'] = None
    settings.CONFIG_DICT['memcached_port'] = None



"""Profile module."""
import yaml
from buzio import console
from tabulate import tabulate
from megalus.core.utils import locale as _


class ProfileMixin():
    """Setting Class."""

    @property
    def has_profile(self):
        """Property: has_profile.

        Check if user data exists.

        Return
        ------
        Bool
        """
        return self.profile is not None

    def edit_profile(self):
        """Function: edit_profile.

        Ask for profile fields to
        populate settings.profile
        """
        if not self.profile:
            self.profile = {
                "cvs_username": "",
                "cvs_password": "",
                "aws_key": "",
                "aws_secret": "",
                "aws_region": "",
                "aws_account": "",
                "slack_url": "",
                "slack_channel": "",
                "slack_icon": "",
                "slack_user": ""
            }
        helptxt = {
            "cvs_username":
                _("Username for {cvs}"),
            "cvs_password": _("Password for {cvs}"),
            "aws_key":
                _("AWS Key"),
            "aws_secret": _("AWS Secret"),
            "aws_region": _("AWS Region"),
            "aws_account": _("AWS account number"),
            "slack_url": _("Slack Webhook URL"),
            "slack_channel": _("Slack Channel"),
            "slack_icon": _("Slack Icon"),
            "slack_user": _("Slack Username")
        }
        key_list = sorted([key for key in self.profile])
        console.section(
            _("{} Profile").format(_("Create")
                                   if not self.profile else _("Edit")),
            full_width=True
        )
        try:
            for key in key_list:
                if "aws" in key and not self.project['use_aws']:
                    continue
                if "slack" in key and not self.project['use_slack']:
                    continue
                if 'cvs' in helptxt[key]:
                    question = helptxt[key].format(
                        cvs=self.project['cvs'].name)
                else:
                    question = helptxt[key]
                self.profile[key] = console.ask(
                    question,
                    default=self.profile[key] if self.profile[key] else None,
                    required=True
                )
        except KeyboardInterrupt:
            return False

        self.show_profile_config()

    def show_profile_config(self):
        """Function: show_profile_config.

        Show profile fields and ask for options
        Edit, Save or Exit
        """
        console.section(_("User Configuration"))
        key_list = sorted([key for key in self.profile])
        table_data = [
            [key, self.profile[key]]
            for key in key_list
            if self.profile[key]
        ]
        print(tabulate(
            table_data,
            headers=[_("Key"), _("Value")],
            tablefmt="presto"
        ))
        print("\n")
        try:
            ret = console.select(
                [
                    _('Edit'),
                    _("Save and Exit"),
                    _("Exit")
                ],
                default=1
            )
            if ret == 0:
                self.edit_profile()
            elif ret == 1:
                self.save_profile()
                console.info(_("Data successfully saved."))
                return True
            else:
                self.profile = None
                return False
        except KeyboardInterrupt:
            return False

    def save_profile(self):
        """Function: save_profile.

        Save settings.profile data
        in separate file on disk.

        Filename is <project slug>_<$USER>.yml
        """
        if self.profile:
            if not self.profilepath:
                self.profilepath = self._get_profile_path()
            with open(self.profilepath, 'w') as file:
                yaml.dump(self.profile, file)



"""[summary].

[description]
"""
import os
import re
from megalus.core import utils
from megalus.projects.config import profile
from megalus.projects.setup import settings
from megalus.core.messages import notify


def run(application, using, rds, verbose):
    """[summary].

    [description]

    Arguments:
        application {[type]} -- [description]
        port {[type]} -- [description]

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False

    container_id, name = utils.get_app(
        application=application,
        title="Rodar Teste",
        data=data
    )

    if not container_id:
        return False

    finalmsg = "Teste não efetuado."
    # Rodar o container com o endereco do
    # Banco de dados selecionado
    dbdata = [
        obj
        for obj in settings.LOCAL_DBS
        if obj.get('admin') == name
    ]
    if rds:
        host = dbdata[0].get('stage_name', None)
        port = dbdata[0].get('stage_port', None)
    else:
        host = dbdata[0].get('local_name', None)
        port = dbdata[0].get('local_port', None)

    if not host or not port:
        utils.console("Nome ou Porta do Banco não encontrado.", style="error")
        return False

    os.chdir(data.project_path)
    # Parar o container
    print(("Rodar Testes - {}".format(name)))
    print("Reiniciando container...")
    utils.run_command(
        get_stdout=True,
        command_list=[
            {
                'command': "cd {} && "
                "docker-compose stop {}".format(
                    data['docker_compose_path'], name),
                'run_stdout': False
            },
        ]
    )

    # Encontrar o programa
    test_app = using if using else data['use_for_tests']
    if not using:
        check_list = [test_app] + settings.TEST_PRIORITY
        ret_pip = utils.run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': "cd {} && "
                    "docker-compose run {} pip freeze".format(
                        data['docker_compose_path'],
                        name),
                    'run_stdout': False}])
        for test in check_list:
            if "{}==".format(test) in ret_pip:
                test_app = test
                break

    # Checa se o coverage está instalado
    dependencies_found = True
    if test_app != "pytest" and 'coverage' not in ret_pip:
        dependencies_found = False
    if 'pydocstyle' not in ret_pip:
        dependencies_found = False
    if 'pycodestyle' not in ret_pip:
        dependencies_found = False
    if test_app == "pytest" and 'pytest-cov' not in ret_pip:
        dependencies_found = False

    # Rodar novo container
    # Para Unittest, Django, Pytest e Nose rodar via Docker-Compose
    new_container_id = utils.run_command(
        get_stdout=True,
        command_list=[
            {
                'command': "cd {} && docker-compose "
                "run -d -e DATABASE_HOST={} -e DATABASE_PORT={} {}".format(
                    data['docker_compose_path'],
                    host,
                    port,
                    name),
                'run_stdout': False},
        ])

    if new_container_id and dependencies_found:

        new_container_id = new_container_id.replace("\n", "")

        database_path = utils.run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': "docker exec -ti {} printenv"
                    " | grep DATABASE_HOST".format(
                        new_container_id),
                    'run_stdout': False
                }
            ]
        )
        os.system('cls' if os.name == 'nt' else 'clear')
        titletext = "Rodando testes com: {}".format(test_app.upper()) + \
            "\nUsando banco de dados: {}".format(
            database_path.replace("\n", "").replace("\r", "").split("=")[1])
        utils.console(titletext, style="main_title", use_prefix=False)

        if test_app == "django":
            command = "coverage run --source='.' manage.py test{}".format(
                ' -v2' if verbose else ""
            )
        elif test_app == "nose":
            command = "nosetests --with-coverage --cover-package=app"
        elif test_app == "pytest":
            command = "pytest --cov=.{}".format(
                " --verbose" if verbose else "")
        else:
            command = "coverage run -m unittest discover -v -s /opt/app"

        os.system(
            'docker exec -ti {} {}'.format(
                new_container_id, command
            )
        )
        # Coverage

        utils.console("Coverage Result", style="section")
        coverage_data = utils.run_command(
            get_stdout=True,
            command_list=[
                {
                    "command": "docker exec -ti {} "
                    "coverage report -m --skip-covered".format(
                        new_container_id),
                    "run_stdout": False
                }
            ]
        )
        print(coverage_data)
        match_groups = re.finditer("(\d+%)", coverage_data)
        min_coverage = os.environ.get('MIN_COVER', 80)
        coverage_total = [
            match.group()
            for match in match_groups
        ][-1]
        coverage_level = int(coverage_total.replace("%", ""))
        coverage_result = True if coverage_level >= min_coverage else False
        if coverage_result:
            utils.console(
                'Result: OK ({}/{})'.format(
                    coverage_level, min_coverage),
                style="success"
            )
        else:
            utils.console(
                'Result: FAIL ({}/{})'.format(
                    coverage_level, min_coverage),
                style="error"
            )

        # PEP8
        pep8_result = utils.run_command(
            title="PEP8 Check",
            get_stdout=False,
            command_list=[
                {
                    "command": "docker exec -ti {} "
                    "pycodestyle {}--exclude=manage.py,settings.py,"
                    "venv,migrations,frontend .".format(
                        new_container_id,
                        "--show-source " if verbose else ""),
                    "run_stdout": False}])

        if pep8_result:
            utils.console('Result: OK', style="success")
        else:
            utils.console('Result: FAIL', style="error")

        # PEP257
        pep257_result = utils.run_command(
            title="PEP257 Check",
            get_stdout=False,
            command_list=[
                {
                    "command": "docker exec -ti {} pydocstyle"
                    " --match-dir='[^venv|^migrations|^frontend].*' "
                    "--match='(?!__|manage).*\.py'".format(
                        new_container_id
                    ),
                    "run_stdout": False
                }
            ]
        )
        if pep257_result:
            utils.console('Result: OK', style="success")
        else:
            utils.console('Result: FAIL', style="error")

        utils.console("Final Result", style="section")
        if coverage_result and pep8_result and pep257_result:
            finalmsg = "Todos os Testes passaram com sucesso."
            utils.console(finalmsg, style="success")
        else:
            finalmsg = "ERROS ENCONTRADOS "
            "DURANTE OS TESTES. Verifique os logs."
            utils.console(finalmsg, style="error")

    elif not dependencies_found:
        utils.console(
            "Verifique se estão instalados no container: "
            "pytest-cov, coverage, pydocstyle ou pycodestyle",
            style="error")
    else:
        utils.console("Nenhum container encontrado", style="error")

    print("\n------------\nReiniciando container...")
    os.system("docker stop {}".format(new_container_id))
    os.system(
        "cd {} && docker-compose "
        "up -d {}".format(
            data['docker_compose_path'], name))

    # Exclui container extra
    # docker rm $(docker ps -a | grep host_run |  awk '{print $1}')
    os.system(
        "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
    )
    notify(msg="Teste Unitário em {}: {}".format(name, finalmsg))
    return False



"""[summary].

[description]
"""
import os
from megalus.core import utils
from megalus.projects.config import profile


def run(application, action, opt=None, arg=None):
    """[summary].

    [description]

    Arguments:
        application {[type]} -- [description]
        action {[type]} -- [description]

    Keyword Arguments:
        opt {[type]} -- [description] (default: {None})
        arg {[type]} -- [description] (default: {None})

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False

    name = ""
    container_id = None
    if application:
        container_id, name = utils.get_app(
            application=application,
            title="Build/Run da Aplicacao",
            data=data,
            stop=False if action == "exec" else True
        )
        if not container_id:
            return False

    if action == "exec":
        utils.console("Rodando comando '{}' em '{}'".format(
            " ".join(arg), name), style="warning", use_prefix=False)
        os.system(
            "docker {cmd} -ti {app}{arg}".format(
                cmd=action,
                app=container_id,
                arg=" {}".format(" ".join(arg)) if arg else "")
        )
    else:
        # Parar os containers
        utils.stop_all(data)

        # Rodar o comando
        os.system(
            "cd {folder} && docker-compose {cmd} {opt} {app}".format(
                folder=data['docker_compose_path'],
                cmd=action,
                app=name,
                opt=opt if opt else "")
        )
    # Exclui container extra
    # docker rm $(docker ps -a | grep host_run |  awk '{print $1}')
    if action == "run":
        os.system(
            "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
        )



import os
import subprocess
import sys
import yaml
from colorama import Fore, Back, Style
from unidecode import unidecode


def get_style(format_type):
    """Function: get_color.

    Summary: return text format color for correpondent type
    Examples: get_color("error")

    Attributes
    ----------
        @param (format_type):string

    Returns
    -------
    Object: Colorama instance

    """
    return {
        'start': Fore.BLUE,
        'error': Fore.RED,
        'success': Fore.GREEN,
        'info': Fore.YELLOW,
        'warning': Fore.YELLOW,
        'section': Fore.YELLOW,
        'git': Back.LIGHTBLUE_EX + Fore.YELLOW,
        'main_title': Fore.BLUE,
        'grey': Fore.LIGHTBLACK_EX
    }.get(format_type, "")


def console(
        text,
        style=None,
        humanize=False,
        format_only=False,
        use_prefix=True,
        bold=False):
    """Function: console.

    Summary: print command replacement using colors
    Examples: console("Hello World!", style="title")

    Attributes
    ----------
        @param (text):String
        @param (style) default=None: String
        @param (humanize) default=False: Bool
        @param (format_only) default=False: Bool
        @param (use_prefix) default=True: Bool
        @param (bold) default=False: Bool

    Returns
    -------
    String or stdout
    """
    if use_prefix and style not in ['section']:
        text = "{}: {}".format(style.upper(), text)
    if humanize:
        text = humanize(text)
    if style == "section":
        line1 = "\n>> {}\n".format(text)
        line2 = "{:*^{num}}\n".format('', num=len(line1) - 2)
        text = line1 + line2
    if style == "main_title":
        line_sizes = [
            len(line)
            for line in text.split("\n")
        ]
        longest_line = sorted(line_sizes, reverse=True)[0]

        horizontal_line = "{:*^{num}}".format('', num=longest_line + 6)
        vertical_line = "*{:^{num}}*".format('', num=longest_line + 4)

        main_texts = [
            "*{:^{num}}*".format(
                text_line, num=longest_line + 4
            )
            for text_line in text.split("\n")
        ]
        text = "{}\n{}\n{}\n{}\n{}".format(
            horizontal_line,
            vertical_line,
            "\n".join(main_texts),
            vertical_line,
            horizontal_line
        )
    if style:
        text = "{}{}{}".format(
            get_style(style),
            text,
            Style.RESET_ALL
        )
    if bold:
        text = "{}{}".format(Style.BRIGHT, text)

    if format_only:
        return text
    else:
        print(text)


def confirma(pergunta, warning=False):
    """Retorna S ou N"""
    resposta_ok = False
    text = console(
        "\n{} (s/n)? ".format(pergunta),
        style="warning" if not warning else "error",
        use_prefix=False,
        format_only=True
    )
    while not resposta_ok:
        resposta = input(text)
        if resposta and resposta[0].upper() in ["S", "N"]:
            resposta_ok = True
    return resposta[0].upper() == "S"


def unitext(text):
    text = unidecode(text)
    return text


def run_command(command_list, title=None, get_stdout=False):
    if title:
        console(title, style="section")
    try:
        for task in command_list:
            if task.get('run_stdout', None):
                command = subprocess.check_output(
                    task['command'],
                    shell=True
                )

                if not command:
                    print('Ocorreu um erro. Processo abortado')
                    return False

                ret = subprocess.call(
                    command,
                    shell=True
                )
            elif get_stdout is True:
                ret = subprocess.check_output(
                    task['command'],
                    shell=True
                )
            else:
                ret = subprocess.call(
                    task['command'],
                    shell=True,
                    stderr=subprocess.STDOUT
                )

            if ret != 0 and not get_stdout:
                # print('Ocorreu um erro. Processo abortado')
                return False
    except BaseException:
        return False

    return True if not get_stdout else ret.decode('utf-8')


def get_app(application, data, title=None, stop=False, no_container=False):
    # 1. Lista todos os containers que estao rodando
    # docker ps -a | grep painel | awk '{print $1,$2}'
    if not stop:
        ret = run_command(
            title=title,
            get_stdout=True,
            command_list=[
                {
                    'command': "docker ps | awk '{print $1, $NF}'",
                    'run_stdout': False
                }
            ]
        )
    else:
        ret = run_command(
            title=title,
            get_stdout=True,
            command_list=[
                {
                    'command': "docker ps -a | awk '{print $1, $NF}'",
                    'run_stdout': False
                }
            ]
        )
        if no_container:
            ret = ret.replace('CONTAINER NAMES\n', '')
            if not ret:
                dc = get_compose_data(data)
                ret = [
                    "0 {}".format(dc['services'][container]['container_name'])
                    for container in dc['services']
                ]
                ret = "\n".join(ret)

    raw_list = ret.split('\n')

    app_list = []

    for obj in raw_list:
        if obj.startswith("CONTAINER"):
            continue
        if len(obj.split(" ")) != 2:
            continue

        app_list.append((
            obj.split(" ")[0],
            obj.split(" ")[1]
        ))

    # 2. Identifica qual o container que bate com o app solicitado
    filtered_list = [
        app
        for app in app_list
        if application and application in app[1]
    ]

    ask_for_app = False
    if filtered_list:
        if len(filtered_list) == 1:
            return (filtered_list[0][0], filtered_list[0][1])
        else:
            ask_for_app = True
    elif app_list:
        ask_for_app = True
    else:
        print("Nenhum aplicativo encontrado.")
        return (None, None)

    if ask_for_app:
        all_apps = filtered_list or app_list
        choices = [
            app[1]
            for app in all_apps
        ]
        rep = escolha(choices)
        return (all_apps[int(rep) - 1][0], all_apps[int(rep) - 1][1])

def escolha(choices):
    i = 1
    for choice in enumerate(choices):
        print("{}. {}".format(i + 1, choice))
    resposta_ok = False
    print("\n")
    while not resposta_ok:
        try:
            rep = input(
                "Selecione (1-{}): ".format(i - 1))
            if rep and int(rep) in range(1, i):
                resposta_ok = True
        except KeyboardInterrupt:
            print("\n")
            return (None, None)
        except BaseException:
            pass
    return rep


def progress_bar(iteration, total, prefix='Lendo',
                 suffix='Complete', barLength=50):
    """
    Gerador de Barra de Progresso
    """
    formatStr = "{0:.2f}"
    percents = formatStr.format(100 * (iteration / float(total)))
    filledLength = int(round(barLength * iteration / float(total)))
    bar = '█' * filledLength + '-' * (barLength - filledLength)
    sys.stdout.write(
        '\r%s |%s| %s%s %s ' %
        (prefix, bar, percents, '%', suffix)),
    sys.stdout.flush()


def get_compose_data(data):
    dc_path = os.path.join(
        data['docker_compose_path'],
        'docker-compose.yml'
    )

    with open(dc_path, 'r') as file:
        dc_data = yaml.load(file)

    return dc_data


def stop_all(data):
    run_command(
        get_stdout=False,
        title=None,
        command_list=[
            {
                'command': "cd {} && docker-compose stop".format(
                    data['docker_compose_path']),
                'run_stdout': False}])
    ret_docker = run_command(
        get_stdout=True,
        command_list=[
            {
                'command': 'docker ps -q',
                'run_stdout': False
            }
        ]
    )
    if ret_docker:
        run_command(
            get_stdout=False,
            command_list=[
                {
                    'command': 'docker stop $(docker ps -q)',
                    'run_stdout': False
                }
            ]
        )


def humanize(data):
    if isinstance(data, str):
        ret = data.replace("_", " ").title()
    if isinstance(data, bool):
        ret = "Sim" if data else "Não"
    if isinstance(data, list) or isinstance(data, tuple):
        ret = ", ".join(data)
    if isinstance(data, dict):
        ret = "\n".join([
            "{}: {}: {}".format(i + 1, key, data[key])
            for i, key in enumerate(data)
        ])

    return ret



"""[summary].

[description]
"""
import os
from megalus.core import utils
from megalus.projects.config import profile


def run(application):
    """[summary].

    [description]

    Arguments:
        application {[type]} -- [description]

    Returns
    -------
        bool -- [description]

    """
    data = profile.get_data()
    if not data:
        return False
    # 1. Identifica o container
    container_id, name = utils.get_app(
        application=application,
        title="Rodar em Modo Depuração",
        data=data)
    if not container_id:
        return False

    # 2. Parar e reiniciar o container com service ports
    # docker-compose stop $app
    # docker-compose run --service-ports $app
    os.system('cls' if os.name == 'nt' else 'clear')
    os.chdir(data.project_path)
    utils.run_command(
        title="Modo Depuração: {}".format(name),
        get_stdout=True,
        command_list=[
            {
                'command': "cd {} && "
                "docker-compose stop {}".format(
                    data['docker_compose_path'], name),
                'run_stdout': False
            },
        ]
    )
    os.system(
        'cd {} && docker-compose run '
        '--service-ports {}\n'.format(
            data['docker_compose_path'], name)
    )

    print("Reiniciando o container...")
    utils.run_command(
        command_list=[
            {'command': "cd {} && "
             "docker-compose up -d {}".format(
                 data['docker_compose_path'], name),
             'run_stdout': False}
        ]
    )

    # Exclui container extra
    # docker rm $(docker ps -a | grep host_run |  awk '{print $1}')
    os.system(
        "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
    )
    return False



import json
import os
import requests
from colorama import Fore, Style
from git import Repo
from requests.auth import HTTPBasicAuth
from megalus.commands.aws import deploy
from megalus.commands.aws.release_eb import AWSManager
from megalus.core.messages import Message, notify
from megalus.core.utils import run_command, confirma, console
from megalus.projects.config import profile
from megalus.projects.setup import settings


def make_pull_request(release):

    data = profile.get_data()
    if not data:
        return False

    # Para criar o pull request para stable
    # é preciso verificar antes:
    console("Release: Gerar/Atualizar Pull Request", style="section")
    # 1. A branch atual é production
    current_dir = os.getcwd()
    folder_name = os.path.split(current_dir)[-1]
    try:
        repo = Repo(current_dir)
        branch = repo.active_branch
    except BaseException:
        print(Fore.RED + "Repositório GIT não encontrado.")
        print("O comando deve ser executado na pasta raiz")
        print("do repositório a ser enviado.")
        print("Comando abortado." + Style.RESET_ALL)
        return False
    if "release" not in branch.name:
        print(
            Fore.RED +
            "Erro: a branch deve ser 'release'" +
            Style.RESET_ALL)
        return False

    # 2. O status git local está OK
    os.chdir(current_dir)
    status_ret = run_command(
        command_list=[
            {
                'command': 'git status -bs --porcelain',
                'run_stdout': False
            }
        ],
        get_stdout=True,
        title=None
    )
    if status_ret.count('\n') > 1 or "[" in status_ret:
        print(Fore.RED + "Erro - a branch está modificada:" + Style.RESET_ALL)
        print(status_ret)
        return False

    # 3. A ultima build da branch atual está OK
    api_repo_url = settings.VCS_BASE_API_URL + \
        settings.REPOSITORY_API_URL.format(
            repo=os.path.split(current_dir)[-1],
            commit=repo.commit().hexsha
        )
    try:
        ret_vcs = requests.get(
            url=api_repo_url,
            auth=HTTPBasicAuth(data['vcs_username'], data['vcs_password']),
            timeout=1
        )
        if ret_vcs.status_code == requests.codes.ok:
            commit_data = ret_vcs.json()
            build_status = commit_data['values'][0]['state']
            if build_status != 'SUCCESSFUL':
                print(Fore.RED + 'Erro: O status da última build é {}'.format(
                    build_status) + Style.RESET_ALL)
                return False
            else:
                print('Última build {}OK{}'.format(
                    Fore.GREEN, Style.RESET_ALL))
        else:
            raise ValueError()
    except BaseException:
        print(
            Fore.RED +
            "Não é possível determinar o status da última build." +
            Style.RESET_ALL)
        return False

    # 4. Gera a tag:
    try:
        last_version = repo.tags[-1].name.split(".")
    except BaseException:
        last_version = [0, 0, 0]

    major = int(last_version[0])
    minor = int(last_version[1])
    patch = int(last_version[2])

    if release == "major":
        major += 1
        minor = 0
        patch = 0
    elif release == "minor":
        minor += 1
        patch = 0
    elif release == "patch":
        patch += 1

    new_version = "{}.{}.{}".format(major, minor, patch)
    print(
        "Versão para o PR: {} -> {}{}{}".format(
            '.'.join(str(e) for e in last_version),
            Fore.YELLOW,
            new_version,
            Style.RESET_ALL))

    # 4. Confirma a operação
    resp = confirma('Deseja continuar')

    if not resp:
        return False

    if release != "same":
        run_command(
            command_list=[
                {
                    'command': 'git tag {}'.format(new_version),
                    'run_stdout': False
                }
            ]
        )

    ret = deploy.run(only_pr=True)

    # 5. Gerar o pull-request
    if ret:
        pr_url = settings.VCS_BASE_API_URL + \
            settings.PULL_REQUEST_API_URL.format(
                repo=os.path.split(current_dir)[-1]
            )
        post_data = {
            "destination": {
                "branch": {
                    "name": "master"
                }
            },
            "source": {
                "branch": {
                    "name": branch.name
                }
            },
            "title": "Pull Request para a versão {}".format(new_version)
        }

        resp = requests.post(
            url=pr_url,
            headers={"Content-Type": "application/json"},
            auth=HTTPBasicAuth(data['vcs_username'], data['vcs_password']),
            data=json.dumps(post_data)
        )

    if resp.status_code == requests.codes.created:
        title = "Pull Request gerado para {}".format(folder_name)
        text = "O usuário {} criou Pull "
        "Request do {} para a release {}".format(
            data['vcs_username'], folder_name, new_version)
        tags = ['Pull Request']

        # Envia Mensagem Datadog/Slack
        message = Message(
            config=data,
            branch=branch.name,
            title=title,
            text=text,
            repo=folder_name
        )
        message.send(alert_type="info", tags=tags)
        print(
            Fore.GREEN +
            "\nPull Request efetuado com sucesso." +
            Style.RESET_ALL)
        notify(msg='Pull Request efetuado com sucesso.')
        return True
    else:
        msg = 'Ocorreu um erro ao tentar enviar o Pull Request'
        print(Fore.RED + msg + Style.RESET_ALL)
        notify(msg=msg)
        return False


def start_deploy():
    if not os.environ.get('BITBUCKET_BRANCH'):
        print(
            Fore.RED +
            "ERRO: Este comando não pode ser executado localmente." +
            Style.RESET_ALL)
        return False

    console("Release: Iniciar Deploy", style="section")
    app_name = os.environ.get('APPLICATION_NAME')
    title = "Deploy Iniciado para".format(app_name)
    text = "Deploy Iniciado para {}".format(
        app_name)
    tags = ['Deploy']

    data = {
        'slack_url': os.environ.get('SLACK_URL'),
        'slack_channel': 'deploy',
        'slack_icon': 'ghost',
        'slack_user': 'bitbucket',
        'datadog_api_key': os.environ.get('DATADOG_API_KEY'),
        'datadog_app_key': os.environ.get('DATADOG_APP_KEY')
    }

    # Envia Mensagem Datadog/Slack
    message = Message(
        config=data,
        branch="master",
        title=title,
        text=text,
        repo=os.environ.get('APP_NAME')
    )
    message.send(alert_type="warning", tags=tags)

    status = AWSManager().deploy()

    if status == 'finished':
        title = "SUCESSO: Deploy finalizado para".format(app_name)
        text = "Deploy finalizado com SUCESSO para {}".format(
            app_name)
        tags = ['Deploy']
        alert_type = 'info'
    elif status == 'failure':
        title = "ERRO: Deploy para".format(app_name)
        text = "Deploy com ERRO para {}".format(
            app_name)
        tags = ['Deploy']
        alert_type = 'warning'
    else:
        title = "EM ANDAMENTO: Deploy para".format(app_name)
        text = "Deploy em andamento para {}. "
        "Por favor acompanhe pelo console do AWS.".format(
            app_name)
        tags = ['Deploy']
        alert_type = 'warning'

    message = Message(
        config=data,
        branch="master",
        title=title,
        text=text,
        repo=os.environ.get('APP_NAME')
    )
    message.send(alert_type=alert_type, tags=tags)

    return True if status != "failure" else False



"""[summary].

[description]
"""
import json
import os
import re
from git import Repo
from megalus.commands.bash import compress
from megalus.core.messages import Message, notify
from megalus.core.utils import run_command, confirma, console
from megalus.projects.config import profile
from megalus.projects.setup import settings


def run(only_pr=False):
    """Function: run.

    Summary: InsertHere
    Examples: InsertHere
    Attributes:
        @param (only_pr) default=False: InsertHere
    Returns: InsertHere
    """
    config = profile.get_data()

    if not config:
        return False

    current_dir = os.getcwd()
    folder_name = os.path.split(current_dir)[-1]
    try:
        repo = Repo(current_dir)
        branch = repo.active_branch
    except BaseException:
        text = [
            "Repositório GIT não encontrado.",
            "O comando deve ser executado na pasta raiz",
            "do repositório a ser enviado.",
            "Comando abortado."
        ]
        console(text, humanize=True, style="error")
        return False
    branch_name = branch.name

    if not only_pr:
        if branch.name == 'master':
            text = " \
                A branch 'master' não \
                pode ser atualizada diretamente,\
                Utilize o comando 'meg ci'. \
                Consulte a ajuda, para mais detalhes."
            console(text, style="error")
            return False

        # Pega a pasta atual e verifica
        # se é uma pasta valida para deploy
        app_list = [
            app.lower()
            for app, br in settings.APPLICATIONS
        ]

        if folder_name.lower() not in app_list:
            console("Repositório não reconhecido", style="error")
            return False

        # Confirma operação
        branch_name = branch.name
        last_commit = repo.head.commit.message
        text_repo = console(folder_name, style="info", format_only=True)
        print("Repositório: {}".format(text_repo))
        if branch_name in ['production', 'master']:
            text_branch = console(
                branch_name.upper(),
                bold=True,
                style="error",
                use_prefix=False,
                format_only=True)
        else:
            text_branch = branch_name.upper()
        print("Branch Atual: {}".format(text_branch))
        text = console(
            last_commit,
            style="warning",
            use_prefix=False,
            format_only=True)
        print("Último Commit:\n{}".format(last_commit))

        # Roda EB Status
        eb_status = False
        ret = run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': "eb status",
                    'run_stdout': False
                }
            ]
        )
        if ret:
            m = re.search("Status: (\w+)", ret)
            if m:
                eb_status_name = m.group(1)
                if eb_status_name == "Ready":
                    eb_status = True
                text = console(
                    eb_status_name.upper(),
                    style="success" if eb_status else "error",
                    use_prefix=False,
                    format_only=True
                )
                print("\nO Status do ElasticBeanstalk é: {}".format(text))
            else:
                console(ret, style="error")

        if not eb_status:
            return False

        resposta = confirma("Confirma o Deploy")
        if not resposta:
            return False

    # Se existir a pasta frontend
    # rodar o build do webpack
    wbpath = os.path.join(current_dir, 'frontend')
    if os.path.exists(wbpath):
        ret = run_command(
            get_stdout=False,
            title="Gerando Build do Webpack",
            command_list=[
                {
                    'command': 'cd {} && ./node_modules/.bin/webpack'
                    ' --config webpack.config.deploy.js'.format(wbpath),
                    'run_stdout': False
                }
            ]
        )
        if not ret:
            return False

    # Ações específicas do App
    # 1. Minify manual
    if folder_name in settings.MINIFY_BEFORE:
        console("Minificando arquivos estáticos", style="section")
        ret = compress.minifyCSS(current_dir=current_dir)
        if not ret:
            return False

        ret = compress.minifyJS(current_dir=current_dir)
        if not ret:
            return False

    # 2. Sincronizar estáticos
    if folder_name in settings.SYNC_S3:
        ret = run_command(
            title="Sincronizando arquivos "
            "estáticos no S3/{}".format(branch_name),
            command_list=[
                {
                    'command': settings.S3_SYNC_CMD.format(
                        branch=branch_name),
                    'run_stdout': False}])
        if not ret:
            return False

    # Gera Dockerrun
    app_name = settings.ECR_NAME.get(folder_name, None)
    if not app_name:
        app_name = folder_name.lower()
    json_model = {
        'AWSEBDockerrunVersion': '1',
        'Image': {
            'Name': '{account}.dkr.ecr.{region}'
            '.amazonaws.com/{app}:{branch}'.format(
                account=config['aws_account'],
                app=app_name,
                branch=branch_name if not only_pr else "master",
                region=config['aws_region']),
            'Update': 'true'},
        'Ports': [
            {
                'ContainerPort': '80'
            }
        ],
        "Volumes": [
            {
                "HostDirectory": "/tmp",
                "ContainerDirectory": "/tmp"
            }
        ],
        'Logging': "/var/eb_log"}

    with open("./Dockerrun.aws.json", 'w') as file:
        file.write(json.dumps(json_model, indent=2))

    if only_pr:
        last_commit = "Pull Request para versão {}".format(
            repo.tags[-1].name)

    ret = run_command(
        title="Adiciona Dockerrun",
        command_list=[
            {
                'command': "git add .",
                'run_stdout': False
            },
            {
                'command': "git commit -m \"{}\"".format(last_commit),
                'run_stdout': False
            }
        ]
    )

    # Atualiza VCS
    ret = run_command(
        title="Atualiza {} - {}".format(settings.VCS_NAME, folder_name),
        command_list=[
            {
                'command': "git push origin {}".format(branch.name),
                'run_stdout': False
            }
        ]
    )
    if not ret:
        return False

    if not only_pr:
        title = "Deploy Iniciado para ".format(folder_name)
        text = "O usuário {} iniciou Deploy do {} para o commit {}".format(
            config['vcs_username'], folder_name, last_commit)
        tags = ['Deploy']

        # Envia Mensagem Datadog/Slack
        if branch.name in ['production', 'master']:
            message = Message(
                config=config,
                branch=branch.name,
                title=title,
                text=text,
                repo=folder_name
            )
            message.send(alert_type="warning", tags=tags)

    # Gerar imagem base do Docker
    dockerbasepath = os.path.join(
        config['project_path'],
        settings.DOCKER_BASE_IMAGE_REPO,
        'baseimage'
    )
    ret = run_command(
        title="Gera Imagem Base do Docker",
        command_list=[
            {
                'command': "aws ecr get-login "
                "--region {region}".format(region=config['aws_region']),
                'run_stdout': True
            },
            {
                'command': "cd {path} && docker build -t {app}:dev .".format(
                    path=dockerbasepath,
                    app=settings.DOCKER_BASE_IMAGE_REPO
                ),
                'run_stdout': False
            },
            {
                'command': "docker tag {base}:dev "
                "{account}.dkr.ecr.{region}.amazonaws.com"
                "/{base}:latest".format(
                    base=settings.DOCKER_BASE_IMAGE_REPO,
                    account=config['aws_account'],
                    region=config['aws_region'],
                ),
                'run_stdout': False
            },
            {
                'command': "docker push {account}."
                "dkr.ecr.{region}.amazonaws.com/{base}:latest".format(
                    account=config['aws_account'],
                    region=config['aws_region'],
                    base=settings.DOCKER_BASE_IMAGE_REPO
                ),
                'run_stdout': False
            },
        ]
    )
    if not ret:
        return False

    # Gerar imagem do Docker
    ret = run_command(
        title="Gera Imagem no Docker - {}".format(folder_name),
        command_list=[
            {
                'command': "docker build -f {name} -t {app}:{branch} .".format(
                    name=settings.DOCKERFILE_DEPLOY,
                    app=app_name,
                    branch=branch_name if not only_pr else "master"
                ),
                'run_stdout': False
            },
            {
                'command': "docker tag {app}:{branch} "
                "{account}.dkr.ecr.{region}.amazonaws.com"
                "/{app}:{branch}".format(
                    account=config['aws_account'],
                    region=config['aws_region'],
                    app=app_name,
                    branch=branch_name if not only_pr else "master"
                ),
                'run_stdout': False
            },
            {
                'command': "docker push {account}."
                "dkr.ecr.{region}.amazonaws.com/{app}:{branch}".format(
                    account=config['aws_account'],
                    region=config['aws_region'],
                    app=app_name,
                    branch=branch_name if not only_pr else "master"
                ),
                'run_stdout': False
            },
        ]
    )
    if not ret:
        return False

    if only_pr:
        return True

    # Rodar EB Deploy
    ret = run_command(
        title="Rodando EB Deploy - {}".format(folder_name),
        command_list=[
            {
                'command': "eb deploy --timeout 60",
                'run_stdout': False
            }
        ]
    )
    if not ret:
        return False

    title = "Deploy Finalizado para ".format(folder_name)
    text = "O Deploy para {} foi finalizado.".format(folder_name)
    tags = ['Deploy']

    # Envia Mensagem Datadog/Slack
    if branch.name in ['production', 'master']:
        message = Message(
            config=config,
            branch=branch.name,
            title=title,
            text=text,
            repo=folder_name
        )
        message.send(alert_type="success", tags=tags)

    notify(msg="O Deploy do {} foi finalizado".format(folder_name))
    return True



"""Settings package for Megalus."""
import os
from buzio import console
from megalus.core.utils import locale as _
from megalus.projects.data import DataMixin
from megalus.projects.profile import ProfileMixin
from megalus.projects.projects import ProjectMixin


class Setting(DataMixin, ProjectMixin, ProfileMixin):
    """Setting Class."""

    def __init__(self, *args, **kwargs):
        """Function: __init__.

        self.compose = data loaded from docker-compose.yml
        self.data = data loaded from ~/.megalus/megconfig.yml
        self.profile = Current User Data Dict
        self.project = Current Project Data Dict

        self.datapath = full path for megconfig.yml
        self.profilepath = full path for project user profile
        """
        self.compose = None
        self.data = {
            'config': {
                'current': None
            },
            'projects': {}
        }
        self.profile = None
        self.project = None

        self.datapath = None
        self.profilepath = None

    @property
    def ready(self):
        """Function: ready.

        Check if data is ready.

        Return
        ------
        Bool
        """
        return self.has_profile and self.has_project

    def check_envs(self):
        """Function: check_envs.

        Check if all environment variables
        listed on settings.project['env_vars']
        are settled in memory.

        Return
        ------
        Bool
        """
        ret = True
        for env in self.project['env_vars']:
            if not os.environ.get(env):
                console.error(
                    _("Define variable '{}' before start.").format(env))
                ret = False
        return ret


settings = Setting()



import re
import os
import subprocess
from buzio import console
from megalus.core.utils import run_command
from megalus.core.utils import locale as _


class DockerBase():

    def __init__(self, settings, arguments):
        self.project = settings.project
        self.profile = settings.profile
        self.arguments = arguments

    def _run_command(self,
                     task,
                     title=None,
                     get_stdout=False,
                     run_stdout=False):

        return run_command(task, title, get_stdout, run_stdout)

    def _get_application(self, container_base=None):

        try:
            # Get container id and name for all running containers
            ret = self._run_command(
                get_stdout=True,
                task="docker ps"
            )
            if not ret:
                console.error(
                    _("Error during task execution. Process aborted."))
                return False, False

            ret = [
                (m.group(1), m.group(2))
                for m in re.finditer("(\w+) .+ (\w.+)", ret)
                if m.group(1) != "CONTAINER"
            ]

            if ret:
                raw_list = ret
            # If there are not running containers
            # get names from project data.
            else:
                raw_list = [
                    (None, app['name'])
                    for app in self.project['dc_applications']
                ]

            # Filter by type, if informed
            container_list = []
            if container_base:
                for index, data in enumerate(raw_list):
                    data_type = [
                        obj['base']
                        for obj in self.project['dc_applications']
                        if obj['name'] == data[1]
                    ][0]
                    if data_type == container_base:
                        container_list.append(data)
            else:
                container_list = raw_list

            # Filter by name if informed
            if self.arguments['<application>']:
                container_list = [
                    obj
                    for obj in container_list
                    if self.arguments['<application>'] in obj[1]
                ]

            if not container_list:
                console.error(_("No valid container/application found."))
                return None, None

            # If one option remains return him
            if len(container_list) == 1:
                return container_list[0]

            # Else choose from list
            else:
                choices = [
                    obj[1]
                    for obj in container_list
                ]
                ret = console.choose(choices)

                return [
                    obj
                    for obj in container_list
                    if obj[1] == ret
                ][0]
        except KeyboardInterrupt:
            return False, False

    def _stop_all_containers(self):
        ret = self._run_command(
            task="cd {} && docker-compose stop".format(
                self.project['dc_path']
            )
        )
        if not ret:
            return False

        check = self._run_command(
            task="docker ps -q",
            get_stdout=True
        )

        if check:
            check = " ".join(check.split("\n"))
            ret = self._run_command(
                task="docker stop {}".format(check)
            )
            if not ret:
                return False

        return True

    def _remove_extra_containers(self):
        ret = self._run_command(
            task="docker ps -a",
            get_stdout=True
        )
        ret = [
            line
            for line in ret
            if "__run__" in line
        ]
        if ret:
            os.system(
                "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
            )



import datadog
import platform
import requests
import slackweb
from colorama import Fore, Style
from megalus.projects.setup import settings
from megalus.core.utils import run_command, unitext, console


class Message():
    def __init__(
            self,
            config,
            title,
            text,
            repo,
            branch,
            test=False,
            *args,
            **kwargs):
        self.config = config
        self.title = title
        self.text = text
        self.test = test
        self.repo = repo
        self.branch = branch
        console("Enviando mensagens", style="section")

    def send(self, alert_type="info", tags=['deploy']):
        if settings.USE_DATADOG:
            self.send_datadog(alert_type, tags=tags)
        if settings.USE_SLACK:
            self.send_slack()
        if settings.USE_GRAFANA:
            self.send_grafana()

    def send_datadog(self, alert_type, tags):

        options = {
            'api_key': self.config['datadog_api_key'],
            'app_key': self.config['datadog_app_key']
        }

        if not self.test:
            datadog.initialize(**options)
            datadog.api.Event.create(
                title=self.title,
                text=self.text,
                tags=tags,
                alert_type=alert_type if not self.test else "info"
            )
            print("Datadog OK")
        else:
            return "{}\n{}".format(self.title, self.text)

    def send_slack(self):

        if not self.test:
            slack = slackweb.Slack(
                url=self.config['slack_url']
            )

            slack.notify(
                text=self.text,
                channel="#{}".format(
                    self.config['slack_channel'])
                if not self.test else settings.TEST_CHANNEL,
                username=self.config['slack_user'],
                icon_emoji=":{}:".format(
                    self.config['slack_icon']))
            print("Slack OK")
        else:
            return self.text

    def send_grafana(self):
        if self.action == "INICIADO":
            status = "started"
            value = 1
        else:
            status = "ended"
            value = 0

        post_data = "{},action=deploy,user={},"
        "repo=megtools,status={},env={} value={}".format(
            self.config['slack_user'],
            self.repo,
            status,
            self.branch,
            value
        )
        if not self.test:
            try:
                ret = requests.post(
                    settings.GRAFANA_MSG_URL, data=post_data, headers={
                        "Content-Type":
                        "application/x-www-form-urlencoded"}, timeout=1)
                if ret.status_code == requests.codes.ok:
                    print("Grafana OK")
                else:
                    print(
                        Fore.RED +
                        "Ocorreu um erro ao "
                        "tentar enviar mensagem ao Grafana" +
                        Style.RESET_ALL)
            except BaseException:
                print(
                    Fore.RED +
                    "Ocorreu um erro ao tentar enviar mensagem ao Grafana" +
                    Style.RESET_ALL)
        else:
            return "Passou pelo Grafana"


def notify(msg, title=None):
    if not title:
        title = "Megalus"
    if platform.system().lower() == 'linux':  # Linux
        run_command(
            command_list=[
                {
                    'command': 'notify-send '
                    '-i gsd-xrandr {title} {msg}'.format(
                        title='"{}"'.format(
                            unitext(title)),
                        msg='"{}"'.format(
                            unitext(msg))),
                    'run_stdout': False}],
            get_stdout=True,
            title=None)
    elif platform.system().lower() != "windows":  # Mac
        run_command(
            command_list=[
                {
                    'command': "osascript -e "
                    "'display notification {msg} "
                    "with title {title}'".format(
                        title='"{}"'.format(
                            unitext(title)),
                        msg='"{}"'.format(
                            unitext(msg))),
                    'run_stdout': False
                }
            ],
            get_stdout=False,
            title=None)



from megalus.command.docker.base import DockerBase
from megalus.core.message import notify
from megalus.core.utils import locale as _
from buzio import console


class Build(DockerBase):

    def run(self):

        console.section("Docker Build", transform="center", full_width=True)

        # Get from name if informed
        name = ""
        container_id = None
        if self.arguments["<application>"]:
            container_id, name = self._get_application(container_base="build")
            if not name:
                return False

        # Login to ECR if used
        if self.project['use_ECR']:
            command = "aws ecr get-login --region {region}".format(
                region=self.profile['aws_region']
            )
            ret = self._run_command(
                task=command,
                run_stdout=True
            )
            if not ret:
                return False

        # Run all base images first - only once per image
        builted_images = []
        for application in self.project['dc_applications']:
            if application['base'] == "build":
                if name and name != application['name']:
                    continue
                if application['base_image_tag'] in builted_images:
                    continue
                if application['build_base_first']:
                    command = "cd {} && docker build -t {}{} .".format(
                        application['base_path'],
                        " --no-cache " if self.arguments["--no-cache"] else "",
                        application['base_image_tag']
                    )
                    ret = self._run_command(
                        title=_("Build base image for {}").format(
                            application['base_image_tag']), task=command)
                    if not ret:
                        return False
                    builted_images.append(application['base_image_tag'])

        # If build only bases, exit
        if self.arguments['--base-only']:
            notify(
                title="Docker Build",
                msg=_("Base image build completed.")
            )
            return True

        # Stop all containers
        ret = self._stop_all_containers()
        if not ret:
            return False

        # Build select application or all
        ret = self._run_command(
            title=_("Building {}").format(name)
            if name else _("Building all containers"),
            task="cd {} && docker-compose build{}{}".format(
                self.project['dc_path'],
                "--no-cache" if self.arguments['--no-cache'] else "",
                " {}".format(name) if name else ""
            )
        )

        # Stop and remove extra containers
        self._stop_all_containers()
        self._remove_extra_containers()

        if ret:
            notify(msg=_("Build operation complete for {}").format(
                name if name else _("all containers")
            )
            )
        else:
            notify(msg=_("An error occured during Docker build."))

        return ret
